"""
Classifies: CHEBI:25409 monoterpenoid
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import rdMolDescriptors

def is_monoterpenoid(smiles: str):
    """
    Determines if a molecule is a monoterpenoid based on having a C10 skeleton derived from monoterpenes,
    potentially modified by removal of methyl groups or other rearrangements.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a monoterpenoid, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Count carbons
    num_carbons = sum(1 for atom in mol.GetAtoms() if atom.GetSymbol() == 'C')
    
    # Basic check for carbon count - monoterpenoids typically have 10 carbons 
    # but can have fewer due to loss of methyl groups
    if num_carbons > 12 or num_carbons < 7:
        return False, f"Carbon count ({num_carbons}) outside typical range for monoterpenoids (7-12)"

    # Check for common monoterpenoid structural features
    
    # Look for cyclic structures (many monoterpenoids are cyclic)
    rings = mol.GetRingInfo()
    has_rings = rings.NumRings() > 0
    
    # Calculate number of unsaturations
    num_unsaturations = rdMolDescriptors.CalcNumRotatableBonds(mol) + rings.NumRings()
    
    # Check for presence of oxygen (common in monoterpenoids)
    has_oxygen = any(atom.GetSymbol() == 'O' for atom in mol.GetAtoms())
    
    # Look for branching (characteristic of terpene structure)
    num_branches = len([atom for atom in mol.GetAtoms() if atom.GetDegree() > 2])
    
    # Combine evidence
    evidence = []
    if has_rings:
        evidence.append("contains rings")
    if num_unsaturations > 0:
        evidence.append(f"contains {num_unsaturations} unsaturations")
    if has_oxygen:
        evidence.append("contains oxygen")
    if num_branches > 0:
        evidence.append(f"contains {num_branches} branches")
        
    # Decision logic
    if num_carbons >= 7 and num_carbons <= 12 and (has_rings or num_unsaturations > 0):
        return True, f"Likely monoterpenoid: {', '.join(evidence)}"
    
    return False, "Does not match monoterpenoid structural patterns"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:25409',
                          'name': 'monoterpenoid',
                          'definition': 'Any terpenoid derived from a '
                                        'monoterpene. The term includes '
                                        'compounds in which the C10 skeleton '
                                        'of the parent monoterpene has been '
                                        'rearranged or modified by the removal '
                                        'of one or more skeletal atoms '
                                        '(generally methyl groups).',
                          'parents': ['CHEBI:26873']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 57,
    'num_false_positives': 100,
    'num_true_negatives': 617,
    'num_false_negatives': 61,
    'num_negatives': None,
    'precision': 0.3630573248407643,
    'recall': 0.4830508474576271,
    'f1': 0.41454545454545455,
    'accuracy': 0.807185628742515}


"""
Classifies: CHEBI:27535 alpha-D-mannoside
"""
from rdkit import Chem
from rdkit.Chem import AllChem, rdMolDescriptors

def is_alpha_D_mannoside(smiles: str):
    """
    Determines if a molecule is an alpha-D-mannoside.

    An alpha-D-mannoside is defined as any mannoside in which the anomeric
    centre has alpha-configuration.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if the molecule is an alpha-D-mannoside, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Find the anomeric carbon
    anomeric_carbon = None
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == 'C' and atom.GetHybridization() == Chem.HybridizationType.SP2:
            for neighbor in atom.GetNeighbors():
                if neighbor.GetSymbol() == 'O' and neighbor.GetDegree() == 2:
                    anomeric_carbon = atom
                    break
            if anomeric_carbon:
                break

    if anomeric_carbon is None:
        return False, "No anomeric carbon found"

    # Check the stereochemistry of the anomeric carbon
    stereo = mol.GetAtomWithIdx(anomeric_carbon.GetIdx()).GetProp('_CIPCode')
    if stereo == 'R':
        alpha_anomer = True
    elif stereo == 'S':
        alpha_anomer = False
    else:
        return False, "Unable to determine stereochemistry of anomeric carbon"

    # Check for mannoside pattern
    mannoside_smarts = 'OC[C@H]1OC(O)[C@@H](O)[C@H](O)[C@@H]1O'
    mannoside_pattern = Chem.MolFromSmarts(mannoside_smarts)
    if mannoside_pattern is None:
        return False, "Error parsing SMARTS pattern"

    if mol.HasSubstructMatch(mannoside_pattern):
        if alpha_anomer:
            return True, "Molecule is an alpha-D-mannoside"
        else:
            return False, "Molecule is a beta-D-mannoside"
    else:
        return False, "Molecule does not contain a mannoside pattern"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:27535',
                          'name': 'alpha-D-mannoside',
                          'definition': 'Any mannoside in which the anomeric '
                                        'centre has alpha-configuration.',
                          'parents': ['CHEBI:25169']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': False,
    'best': True,
    'error': "'_CIPCode'",
    'stdout': '',
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 0,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0,
    'f1': 0,
    'accuracy': None}


"""
Classifies: CHEBI:21524 N-acetyl-D-glucosaminyl groups
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import rdMolDescriptors

def is_N_acetyl_D_glucosaminyl_groups(smiles: str):
    """
    Determines if a molecule contains N-acetyl-D-glucosaminyl groups.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule contains N-acetyl-D-glucosaminyl groups, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # SMARTS pattern for N-acetyl-D-glucosaminyl core
    # Matches pyranose ring with N-acetyl group and correct stereochemistry
    pattern = "[C@H]1[C@H]([C@H](O)[C@H]([CH]O1)NC(C)=O)O"
    
    substructure = Chem.MolFromSmarts(pattern)
    if substructure is None:
        return None, "Invalid SMARTS pattern"
        
    # Find matches
    matches = mol.GetSubstructMatches(substructure)
    
    if not matches:
        return False, "No N-acetyl-D-glucosaminyl groups found"
        
    # Check for pyranose ring
    rings = mol.GetRingInfo()
    if not any(len(ring) == 6 for ring in rings.AtomRings()):
        return False, "No 6-membered pyranose ring found"
        
    # Verify key structural features
    for match in matches:
        # Get atoms in match
        atoms = [mol.GetAtomWithIdx(i) for i in match]
        
        # Check for required functional groups
        has_n_acetyl = False
        has_hydroxyls = False
        
        for atom in atoms:
            # Check for N-acetyl group
            if atom.GetSymbol() == 'N':
                for neighbor in atom.GetNeighbors():
                    if neighbor.GetSymbol() == 'C':
                        for n2 in neighbor.GetNeighbors():
                            if n2.GetSymbol() == 'O' and n2.GetIsAromatic() == False:
                                has_n_acetyl = True
                                
            # Check for hydroxyl groups
            if atom.GetSymbol() == 'O' and atom.GetTotalNumHs() > 0:
                has_hydroxyls = True
                
        if not (has_n_acetyl and has_hydroxyls):
            continue
            
        return True, "Contains N-acetyl-D-glucosaminyl group(s)"
        
    return False, "Structure does not match N-acetyl-D-glucosaminyl group requirements"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:21524',
                          'name': 'N-acetyl-D-glucosaminyl groups',
                          'definition': 'A glucosaminyl group having '
                                        'D-configuration and an N-acetyl '
                                        'substituent (and derivatives '
                                        'thereof).',
                          'parents': ['CHEBI:24272']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 5,
    'num_false_positives': 100,
    'num_true_negatives': 1474,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.047619047619047616,
    'recall': 1.0,
    'f1': 0.0909090909090909,
    'accuracy': 0.9366687777074097}


"""
Classifies: CHEBI:2571 aliphatic alcohol
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import rdMolDescriptors

def is_aliphatic_alcohol(smiles: str):
    """
    Determines if a molecule contains an aliphatic alcohol group.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule contains aliphatic alcohol, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"
    
    # Find all OH groups
    oh_matches = mol.GetSubstructMatches(Chem.MolFromSmarts('[OH1]'))
    if not oh_matches:
        return False, "No hydroxyl groups found"
        
    # For each OH group, check if it's attached to an aliphatic carbon
    for match in oh_matches:
        oh_atom = mol.GetAtomWithIdx(match[0])
        carbon_neighbor = None
        
        # Get the carbon atom attached to OH
        for neighbor in oh_atom.GetNeighbors():
            if neighbor.GetSymbol() == 'C':
                carbon_neighbor = neighbor
                break
                
        if carbon_neighbor is None:
            continue
            
        # Check if carbon is aliphatic (not aromatic)
        if not carbon_neighbor.GetIsAromatic():
            # Count the number of aliphatic alcohols found
            n_aliphatic_oh = sum(1 for m in oh_matches 
                               if not mol.GetAtomWithIdx(m[0]).GetNeighbors()[0].GetIsAromatic())
            return True, f"Contains {n_aliphatic_oh} aliphatic hydroxyl group(s)"
            
    return False, "No aliphatic hydroxyl groups found"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:2571',
                          'name': 'aliphatic alcohol',
                          'definition': 'An  alcohol derived from an aliphatic '
                                        'compound.',
                          'parents': ['CHEBI:30879']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': "Attempt failed: 'Atom' object has no attribute "
               "'GetIsConjugated'",
    'attempt': 1,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 90,
    'num_false_positives': 100,
    'num_true_negatives': 73,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.47368421052631576,
    'recall': 1.0,
    'f1': 0.6428571428571429,
    'accuracy': 0.6197718631178707}


"""
Classifies: CHEBI:24709 hydroxylamines
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_hydroxylamines(smiles: str):
    """
    Determines if a molecule is a hydroxylamine (H2N-OH) or its hydrocarbyl derivatives.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a hydroxylamine, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Look for N-O single bonds
    n_o_bonds = []
    for bond in mol.GetBonds():
        if (bond.GetBeginAtom().GetSymbol() == 'N' and bond.GetEndAtom().GetSymbol() == 'O') or \
           (bond.GetBeginAtom().GetSymbol() == 'O' and bond.GetEndAtom().GetSymbol() == 'N'):
            if bond.GetBondType() == Chem.BondType.SINGLE:
                n_o_bonds.append(bond)

    if not n_o_bonds:
        return False, "No N-O single bonds found"

    # Check each N-O bond to confirm hydroxylamine pattern
    for bond in n_o_bonds:
        n_atom = bond.GetBeginAtom() if bond.GetBeginAtom().GetSymbol() == 'N' else bond.GetEndAtom()
        o_atom = bond.GetEndAtom() if bond.GetEndAtom().GetSymbol() == 'O' else bond.GetBeginAtom()
        
        # Check oxygen atom
        if len(o_atom.GetBonds()) != 1:  # O should only have one bond
            continue
            
        # Check nitrogen atom
        n_bonds = len(n_atom.GetBonds())
        if n_bonds < 2 or n_bonds > 3:  # N should have 2 or 3 bonds
            continue
            
        # Check if N is part of an amide (N-C(=O))
        is_amide = False
        for neighbor in n_atom.GetNeighbors():
            if neighbor.GetSymbol() == 'C':
                for c_neighbor in neighbor.GetNeighbors():
                    if c_neighbor.GetSymbol() == 'O' and \
                       mol.GetBondBetweenAtoms(neighbor.GetIdx(), c_neighbor.GetIdx()).GetBondType() == Chem.BondType.DOUBLE:
                        is_amide = True
                        break
                        
        if is_amide:
            continue

        # If we get here, we've found a valid hydroxylamine group
        if n_bonds == 2:
            return True, "Primary hydroxylamine (H2N-OH) derivative found"
        else:
            return True, "Secondary hydroxylamine (RHN-OH) or tertiary hydroxylamine (R2N-OH) derivative found"

    return False, "No hydroxylamine groups found"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:24709',
                          'name': 'hydroxylamines',
                          'definition': 'Hydroxylamine, H2N-OH, and its '
                                        'hydrocarbyl derivatives.',
                          'parents': ['CHEBI:51143']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 7,
    'num_false_positives': 100,
    'num_true_negatives': 7293,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.06542056074766354,
    'recall': 1.0,
    'f1': 0.12280701754385964,
    'accuracy': 0.9864864864864865}


"""
Classifies: CHEBI:176843 vitamin B12
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_vitamin_B12(smiles: str):
    """
    Determines if a molecule is a vitamin B12 or one of its vitamers.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a vitamin B12 vitamer, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check for the presence of a cobalt atom
    has_cobalt = False
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == 'Co':
            has_cobalt = True
            break

    if not has_cobalt:
        return False, "No cobalt atom found"

    # Check for the corrin ring system
    corrin_ring_atoms = []
    corrin_ring_bond_orders = []

    # Define the expected corrin ring atom symbols and bond orders
    expected_corrin_ring_atoms = ['C', 'C', 'C', 'N', 'C', 'C', 'C', 'N', 'C', 'C', 'C', 'N', 'C', 'C', 'C', 'N']
    expected_corrin_ring_bond_orders = [1, 2, 1, 1, 2, 1, 1, 1, 2, 1, 1, 1, 2, 1, 1, 1]

    # Find the corrin ring system
    for ring in mol.GetRingInfo().AtomRings():
        if len(ring) == 16:
            ring_atoms = [mol.GetAtomWithIdx(idx) for idx in ring]
            ring_bond_orders = [mol.GetBondBetweenAtoms(ring_atoms[i].GetIdx(), ring_atoms[(i + 1) % 16].GetIdx()).GetBondType() for i in range(16)]

            if [atom.GetSymbol() for atom in ring_atoms] == expected_corrin_ring_atoms and ring_bond_orders == expected_corrin_ring_bond_orders:
                corrin_ring_atoms = ring_atoms
                corrin_ring_bond_orders = ring_bond_orders
                break

    if not corrin_ring_atoms:
        return False, "No corrin ring system found"

    # Check for the presence of substituents characteristic of vitamin B12 vitamers
    substituents = []
    for atom in mol.GetAtoms():
        if atom not in corrin_ring_atoms:
            symbol = atom.GetSymbol()
            if symbol in ['N', 'O', 'C']:
                substituents.append(symbol)

    if 'N' in substituents and 'O' in substituents and 'C' in substituents:
        return True, "Molecule is a vitamin B12 vitamer"
    else:
        return False, "Substituents not characteristic of vitamin B12 vitamers"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:176843',
                          'name': 'vitamin B12',
                          'definition': 'Any member of a group of cobalamin '
                                        'vitamers that exhibit biological '
                                        'activity against vitamin B12 '
                                        'deficiency. Vitamin B12 deficiency is '
                                        'associated with low red blood cell '
                                        'count and anemia. The vitamers are '
                                        'found in foods such as cereals, meat, '
                                        'fish, and poultry. The vitamers '
                                        'include adenosylcobalamin, '
                                        'hydroxocobalamin, cyanocobalamin, '
                                        'aquacobalamin, nitritocobalamin and '
                                        'methylcobabalamin (also includes '
                                        'their ionized, salt and phosphate '
                                        'derivatives).',
                          'parents': ['CHEBI:75769']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 183924,
    'num_false_negatives': 1,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.9999945630012234}


"""
Classifies: CHEBI:21545 N-acetyl-L-amino acid
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem.AllChem import GetMorganFingerprintAsBitVect

def is_N_acetyl_L_amino_acid(smiles: str):
    """
    Determines if a molecule is an N-acetyl-L-amino acid.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is an N-acetyl-L-amino acid, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"
        
    # Check for carboxylic acid group
    carboxyl_pattern = Chem.MolFromSmarts('C(=O)[OH]')
    if not mol.HasSubstructMatch(carboxyl_pattern):
        return False, "No carboxylic acid group found"
        
    # Check for N-acetyl group
    n_acetyl_pattern = Chem.MolFromSmarts('NC(=O)C')
    if not mol.HasSubstructMatch(n_acetyl_pattern):
        return False, "No N-acetyl group found"
        
    # Check for alpha carbon with correct stereochemistry
    alpha_carbon_pattern = Chem.MolFromSmarts('[C@@H](N)(C(=O)O)C')
    alpha_carbon_pattern_alt = Chem.MolFromSmarts('[C@H](N)(C(=O)O)C') 
    
    if not (mol.HasSubstructMatch(alpha_carbon_pattern) or mol.HasSubstructMatch(alpha_carbon_pattern_alt)):
        return False, "No alpha carbon with correct stereochemistry found"
        
    # Additional check to ensure N-acetyl is connected to alpha carbon
    matches = mol.GetSubstructMatches(Chem.MolFromSmarts('CC(=O)N[C@@H](C)C(O)=O'))
    matches_alt = mol.GetSubstructMatches(Chem.MolFromSmarts('CC(=O)N[C@H](C)C(O)=O'))
    
    if not (matches or matches_alt):
        return False, "N-acetyl group not properly connected to alpha carbon"
    
    # If all checks pass, it's an N-acetyl-L-amino acid
    return True, "Molecule contains N-acetyl group, carboxylic acid, and correct stereochemistry"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:21545',
                          'name': 'N-acetyl-L-amino acid',
                          'definition': 'An L-amino acid having an N-acetyl '
                                        'substituent.',
                          'parents': [   'CHEBI:21575',
                                         'CHEBI:21644',
                                         'CHEBI:22160']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 2,
    'num_false_positives': 100,
    'num_true_negatives': 1479,
    'num_false_negatives': 1,
    'num_negatives': None,
    'precision': 0.0196078431372549,
    'recall': 0.6666666666666666,
    'f1': 0.03809523809523809,
    'accuracy': 0.9361567635903919}


"""
Classifies: CHEBI:33694 biomacromolecule
"""
from rdkit import Chem
from rdkit.Chem import AllChem, Descriptors
from rdkit.Chem import rdMolDescriptors

def is_biomacromolecule(smiles: str):
    """
    Determines if a molecule is a biomacromolecule.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a biomacromolecule, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check if molecule contains only C, H, O, N, P, S atoms
    allowed_atoms = set('CHONPS')
    atom_counts = Chem.Mol.GetAtomCountDict(mol)
    if any(atom not in allowed_atoms for atom in atom_counts):
        return False, "Molecule contains atoms other than C, H, O, N, P, S"

    # Check for common biomacromolecular substructures
    biomacro_patterns = [
        Chem.MolFromSmarts("OC(=O)C"),  # Carboxylic acid
        Chem.MolFromSmarts("NC(=O)C"),  # Amide
        Chem.MolFromSmarts("OC(=O)N"),  # Carbamate
        Chem.MolFromSmarts("OP(=O)(O)O"),  # Phosphate
        Chem.MolFromSmarts("O[C@H]1[C@H](O)[C@@H](O)[C@H](O)[C@@H]1O"),  # Pyranose
        Chem.MolFromSmarts("C1=CN=CN=C1"),  # Purine
        Chem.MolFromSmarts("C1=CNC=N1"),  # Pyrimidine
    ]

    if not any(mol.HasSubstructMatch(pattern) for pattern in biomacro_patterns):
        return False, "No common biomacromolecular substructures found"

    # Check for high molecular weight
    mw = Descriptors.MolWt(mol)
    if mw < 500:
        return False, "Molecular weight too low for a macromolecule"

    return True, "Molecule is classified as a biomacromolecule"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:33694',
                          'name': 'biomacromolecule',
                          'definition': 'A macromolecule formed by a living '
                                        'organism.',
                          'parents': ['CHEBI:33839', 'CHEBI:50860']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': False,
    'best': True,
    'error': "type object 'Mol' has no attribute 'GetAtomCountDict'",
    'stdout': '',
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 0,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0,
    'f1': 0,
    'accuracy': None}


"""
Classifies: CHEBI:33470 heteroaryl group
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_heteroaryl_group(smiles: str):
    """
    Determines if a molecule is a heteroaryl group, defined as a heterocyclyl group derived
    from a heteroarene by removal of a hydrogen atom from any ring atom.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a heteroaryl group, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check if the molecule has at least one aromatic ring
    aromatic_rings = AllChem.GetAromaticRings(mol)
    if not aromatic_rings:
        return False, "No aromatic rings found"

    # Check if the molecule has at least one heteroatom in an aromatic ring
    heteroatom_rings = []
    for ring in aromatic_rings:
        atoms = [mol.GetAtomWithIdx(idx) for idx in ring]
        if any(atom.GetAtomicNum() != 6 for atom in atoms):
            heteroatom_rings.append(ring)

    if not heteroatom_rings:
        return False, "No aromatic rings with heteroatoms found"

    # Check if the molecule has a radical site
    radical_atom_idx = None
    for atom in mol.GetAtoms():
        if atom.GetNumRadicalElectrons() > 0:
            radical_atom_idx = atom.GetIdx()
            break

    if radical_atom_idx is None:
        return False, "No radical site found"

    # Check if the radical site is in an aromatic ring with heteroatoms
    for ring in heteroatom_rings:
        if radical_atom_idx in ring:
            return True, "Heteroaryl group detected"

    return False, "Radical site is not in a heteroaromatic ring"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:33470',
                          'name': 'heteroaryl group',
                          'definition': 'A heterocyclyl group derived from a '
                                        'heteroarene by removal of a hydrogen '
                                        'atom from any ring atom.',
                          'parents': ['CHEBI:33453']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': False,
    'best': True,
    'error': "module 'rdkit.Chem.AllChem' has no attribute 'GetAromaticRings'",
    'stdout': '',
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 0,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0,
    'f1': 0,
    'accuracy': None}


"""
Classifies: CHEBI:26333 prostaglandin
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem.rdMolDescriptors import CalcMolFormula

def is_prostaglandin(smiles: str):
    """
    Determines if a molecule is a prostaglandin based on structural features.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a prostaglandin, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"
        
    # Check carbon count - should be C20 or close
    formula = CalcMolFormula(mol)
    carbon_count = sum(1 for c in formula if c == 'C')
    if carbon_count < 18 or carbon_count > 22:
        return False, f"Carbon count {carbon_count} outside typical range for prostaglandins (18-22)"

    # Check for carboxylic acid group
    carboxylic_acid_pattern = Chem.MolFromSmarts('C(=O)[OH]')
    if not mol.HasSubstructMatch(carboxylic_acid_pattern):
        return False, "Missing carboxylic acid group characteristic of prostaglandins"

    # Check for cyclopentane ring
    cyclopentane_pattern = Chem.MolFromSmarts('C1CCCC1')
    if not mol.HasSubstructMatch(cyclopentane_pattern):
        return False, "Missing cyclopentane ring characteristic of prostaglandins"

    # Check for hydroxyl groups (most prostaglandins have at least one)
    hydroxyl_pattern = Chem.MolFromSmarts('[OH]')
    if not mol.HasSubstructMatch(hydroxyl_pattern):
        return False, "Missing hydroxyl groups typically found in prostaglandins"

    # Check for alkene groups (most prostaglandins have at least one)
    alkene_pattern = Chem.MolFromSmarts('C=C')
    if not mol.HasSubstructMatch(alkene_pattern):
        return False, "Missing alkene groups typically found in prostaglandins"

    # If all basic structural features are present, classify as prostaglandin
    features = []
    if mol.HasSubstructMatch(Chem.MolFromSmarts('C(=O)[OH]')):
        features.append("carboxylic acid")
    if mol.HasSubstructMatch(Chem.MolFromSmarts('C1CCCC1')):
        features.append("cyclopentane ring")
    if mol.HasSubstructMatch(Chem.MolFromSmarts('[OH]')):
        num_hydroxyls = len(mol.GetSubstructMatches(hydroxyl_pattern))
        features.append(f"{num_hydroxyls} hydroxyl groups")
    if mol.HasSubstructMatch(Chem.MolFromSmarts('C=C')):
        num_alkenes = len(mol.GetSubstructMatches(alkene_pattern))
        features.append(f"{num_alkenes} alkene groups")
        
    return True, f"Prostaglandin with {', '.join(features)}"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:26333',
                          'name': 'prostaglandin',
                          'definition': 'Naturally occurring compounds derived '
                                        'from the parent C20 acid, prostanoic '
                                        'acid.',
                          'parents': ['CHEBI:26347']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 183841,
    'num_false_negatives': 9,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.9999510470492249}


"""
Classifies: CHEBI:28963 amino sugar
"""
I can analyze the chemical structures provided in the SMILES strings to classify them into different classes of chemical entities. However, it would be helpful if you could provide some additional context or specific requirements for the classification task. This will help me provide a more accurate and relevant analysis.

For example, are you interested in classifying these compounds based on their functional groups, molecular scaffolds, biological activities, or any other specific criteria? Providing this information will allow me to tailor my analysis and classification accordingly.


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:28963',
                          'name': 'amino sugar',
                          'definition': 'Any sugar having one or more '
                                        'alcoholic hydroxy groups replaced by '
                                        'substituted or unsubstituted amino '
                                        'groups.',
                          'parents': ['CHEBI:63299']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': False,
    'best': True,
    'error': 'invalid syntax (<string>, line 1)',
    'stdout': '',
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 0,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': None}


"""
Classifies: CHEBI:24261 glucocorticoid
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem.AllChem import GetMorganFingerprintAsBitVect
from rdkit.Chem import rdMolDescriptors

def is_glucocorticoid(smiles: str):
    """
    Determines if a molecule is a glucocorticoid based on structural features.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a glucocorticoid, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Check for presence of steroid core (4 fused rings)
    rings = mol.GetRingInfo()
    if len(rings.AtomRings()) < 4:
        return False, "Missing steroid core (4 fused rings)"
    
    # Check for 6-6-6-5 ring pattern characteristic of steroids
    ring_sizes = [len(ring) for ring in rings.AtomRings()]
    if not (ring_sizes.count(6) >= 3 and 5 in ring_sizes):
        return False, "Missing characteristic 6-6-6-5 steroid ring pattern"

    # Check for key functional groups common in glucocorticoids
    # Look for ketone at C3 position (typically in A ring)
    patt_ketone = Chem.MolFromSmarts('C1CCC(=O)CC1')
    if not mol.HasSubstructMatch(patt_ketone):
        return False, "Missing ketone group in ring A"

    # Look for hydroxyl groups (common in glucocorticoids)
    patt_oh = Chem.MolFromSmarts('OH')
    if not mol.HasSubstructMatch(patt_oh):
        return False, "Missing hydroxyl groups"

    # Look for C17 side chain with hydroxyl/ketone
    patt_c17_chain = Chem.MolFromSmarts('[CH2]C(=O)CO')
    patt_c17_chain_alt = Chem.MolFromSmarts('[CH2]C(O)CO')
    
    if not (mol.HasSubstructMatch(patt_c17_chain) or mol.HasSubstructMatch(patt_c17_chain_alt)):
        return False, "Missing characteristic C17 side chain"

    # If all checks pass, likely a glucocorticoid
    # Identify some common modifications
    modifications = []
    
    # Check for halogen substitutions
    if mol.HasSubstructMatch(Chem.MolFromSmarts('[F,Cl,Br,I]')):
        modifications.append("halogenated")
        
    # Check for acetate groups
    if mol.HasSubstructMatch(Chem.MolFromSmarts('CC(=O)O')):
        modifications.append("acetylated")
        
    # Check for methyl groups beyond the core structure
    if len(mol.GetSubstructMatches(Chem.MolFromSmarts('C[CH3]'))) > 2:
        modifications.append("additional methyl groups")

    if modifications:
        return True, f"Glucocorticoid with modifications: {', '.join(modifications)}"
    else:
        return True, "Basic glucocorticoid structure"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:24261',
                          'name': 'glucocorticoid',
                          'definition': 'Glucocorticoids are a class of '
                                        'steroid hormones that regulate a '
                                        'variety of physiological processes, '
                                        'in particular control of the '
                                        'concentration of glucose in blood.',
                          'parents': ['CHEBI:36699']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': False,
    'best': True,
    'error': 'Python argument types in\n'
             '    Mol.HasSubstructMatch(Mol, NoneType)\n'
             'did not match C++ signature:\n'
             '    HasSubstructMatch(RDKit::ROMol self, RDKit::MolBundle query, '
             'RDKit::SubstructMatchParameters params=True)\n'
             '    HasSubstructMatch(RDKit::ROMol self, RDKit::ROMol query, '
             'RDKit::SubstructMatchParameters params)\n'
             '    HasSubstructMatch(RDKit::ROMol self, RDKit::MolBundle query, '
             'bool recursionPossible=True, bool useChirality=False, bool '
             'useQueryQueryMatches=False)\n'
             '    HasSubstructMatch(RDKit::ROMol self, RDKit::ROMol query, '
             'bool recursionPossible=True, bool useChirality=False, bool '
             'useQueryQueryMatches=False)',
    'stdout': '',
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 0,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': None}


"""
Classifies: CHEBI:22736 benzoyl-CoAs
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_benzoyl_CoAs(smiles: str):
    """
    Determines if a molecule is a benzoyl-CoA, defined as any aroyl-CoA in which the aroyl group is specified as benzoyl or its substituted derivative.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a benzoyl-CoA, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check for the presence of a CoA moiety
    coa_pattern = Chem.MolFromSmarts('C(=O)NCCC(=O)NCCSCCNC(=O)CCNC(=O)C')
    if not mol.HasSubstructMatch(coa_pattern):
        return False, "CoA moiety not found"

    # Find the benzoyl group
    benzoyl_pattern = Chem.MolFromSmarts('C(=O)C1=CC=CC=C1')
    benzoyl_match = mol.GetSubstructMatches(benzoyl_pattern)

    if not benzoyl_match:
        return False, "Benzoyl group not found"

    # Check for substitutions on the benzoyl group
    benzoyl_ring = Chem.GetMolFrags(mol)[benzoyl_match[0]]
    substituents = []
    for atom in benzoyl_ring:
        if mol.GetAtomWithIdx(atom).GetSymbol() != 'C' and mol.GetAtomWithIdx(atom).GetSymbol() != 'H':
            substituents.append(mol.GetAtomWithIdx(atom).GetSymbol())

    if len(substituents) > 0:
        return True, f"Substituted benzoyl-CoA with substituents: {', '.join(set(substituents))}"
    else:
        return True, "Unsubstituted benzoyl-CoA"

# Example usage
smiles = "[C@@H]1(N2C3=C(C(=NC=N3)N)N=C2)O[C@H](COP(OP(OCC([C@H](C(NCCC(NCCSC(=O)C=4C=CC(=C(C4)OC)O)=O)=O)O)(C)C)(=O)O)(=O)O)[C@H]([C@H]1O)OP(O)(O)=O"
result, reason = is_benzoyl_CoAs(smiles)
print(result, reason)


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:22736',
                          'name': 'benzoyl-CoAs',
                          'definition': 'Any  aroyl-CoA in which the  aroyl '
                                        'group is specified as benzoyl or its '
                                        'substituted derivative.',
                          'parents': ['CHEBI:61940']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 183924,
    'num_false_negatives': 1,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0,
    'accuracy': 0.9999945630012234}


"""
Classifies: CHEBI:26607 saturated fatty acid
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_saturated_fatty_acid(smiles: str):
    """
    Determines if a molecule is a saturated fatty acid.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a saturated fatty acid, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"
        
    # Check for carboxylic acid group
    carboxylic_pattern = Chem.MolFromSmarts('C(=O)[OH]')
    if not mol.HasSubstructMatch(carboxylic_pattern):
        return False, "No carboxylic acid group found"
        
    # Check for presence of carbon chain
    carbon_chain = False
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == 'C':
            carbon_chain = True
            break
    if not carbon_chain:
        return False, "No carbon chain found"
        
    # Check for absence of multiple bonds
    for bond in mol.GetBonds():
        if bond.GetBondType() not in [Chem.BondType.SINGLE]:
            if not (bond.GetBeginAtom().GetSymbol() == 'O' and 
                   bond.GetEndAtom().GetSymbol() == 'C' and
                   bond.GetBondType() == Chem.BondType.DOUBLE):  # Allow C=O bond
                return False, "Contains carbon-carbon multiple bonds"
                
    # Count carbons in main chain
    carbon_count = sum(1 for atom in mol.GetAtoms() if atom.GetSymbol() == 'C')
    
    # Check if molecule contains only C, H, O
    valid_atoms = {'C', 'H', 'O'}
    for atom in mol.GetAtoms():
        if atom.GetSymbol() not in valid_atoms:
            return False, f"Contains non C/H/O atom: {atom.GetSymbol()}"
            
    # Success - classify based on chain length
    if carbon_count < 2:
        return False, "Carbon chain too short to be a fatty acid"
    elif carbon_count <= 4:
        return True, "Short-chain saturated fatty acid"
    elif carbon_count <= 12:
        return True, "Medium-chain saturated fatty acid"
    else:
        return True, "Long-chain saturated fatty acid"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:26607',
                          'name': 'saturated fatty acid',
                          'definition': 'Any fatty acid containing no carbon '
                                        'to carbon multiple bonds. Known to '
                                        'produce adverse biological effects '
                                        'when ingested to excess.',
                          'parents': ['CHEBI:35366']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 89,
    'num_true_negatives': 183742,
    'num_false_negatives': 10,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.9994614911798783}


"""
Classifies: CHEBI:18946 delta-lactone
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_delta_lactone(smiles: str):
    """
    Determines if a molecule is a delta-lactone (6-membered lactone ring).
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a delta-lactone, False otherwise
        str: Reason for classification
    """
    
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Generate ring info
    rings = mol.GetRingInfo()
    
    # Find all 6-membered rings
    six_rings = []
    for ring in rings.AtomRings():
        if len(ring) == 6:
            six_rings.append(ring)
            
    if not six_rings:
        return False, "No 6-membered rings found"
        
    # Check each 6-membered ring for lactone pattern
    for ring in six_rings:
        ring_atoms = [mol.GetAtomWithIdx(i) for i in ring]
        
        # Look for O-C(=O) pattern in ring
        for i, atom in enumerate(ring_atoms):
            if atom.GetSymbol() == 'O':
                # Get next atom in ring
                next_idx = (i + 1) % 6
                next_atom = ring_atoms[next_idx]
                
                if next_atom.GetSymbol() == 'C':
                    # Check if carbon is double bonded to oxygen
                    for bond in next_atom.GetBonds():
                        other_atom = bond.GetOtherAtom(next_atom)
                        if (other_atom.GetSymbol() == 'O' and 
                            bond.GetBondType() == Chem.rdchem.BondType.DOUBLE and
                            other_atom.GetIdx() not in ring):
                            return True, "Contains 6-membered lactone ring"
                            
    return False, "No lactone pattern found in 6-membered rings"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:18946',
                          'name': 'delta-lactone',
                          'definition': 'A lactone having a six-membered '
                                        'lactone ring.',
                          'parents': ['CHEBI:25000']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 49,
    'num_false_positives': 100,
    'num_true_negatives': 5446,
    'num_false_negatives': 26,
    'num_negatives': None,
    'precision': 0.3288590604026846,
    'recall': 0.6533333333333333,
    'f1': 0.43749999999999994,
    'accuracy': 0.9775840597758406}


"""
Classifies: CHEBI:25883 pentahydroxyflavone
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import rdMolDescriptors

def is_pentahydroxyflavone(smiles: str):
    """
    Determines if a molecule is a pentahydroxyflavone (flavone with 5 hydroxy groups).
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a pentahydroxyflavone, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Check for flavone core structure
    flavone_pattern = Chem.MolFromSmarts('[#6]1=[#6]-c2c([#6](=[O])[#6]1)c(=O)c1ccccc1o2')
    if not mol.HasSubstructMatch(flavone_pattern):
        return False, "No flavone core structure found"
    
    # Count number of hydroxy groups directly attached to the flavone core
    # First get the atoms in the flavone core
    flavone_match = mol.GetSubstructMatch(flavone_pattern)
    flavone_atoms = set(flavone_match)
    
    # Pattern for hydroxy groups
    oh_pattern = Chem.MolFromSmarts('O[H]')
    
    # Count hydroxy groups attached to flavone core
    hydroxy_count = 0
    matches = mol.GetSubstructMatches(oh_pattern)
    
    for match in matches:
        oh_oxygen = match[0]
        # Get the atom the OH is attached to
        for bond in mol.GetAtomWithIdx(oh_oxygen).GetBonds():
            neighbor = bond.GetOtherAtom(mol.GetAtomWithIdx(oh_oxygen))
            # Check if this neighbor is part of the flavone core or directly attached to it
            if neighbor.GetIdx() in flavone_atoms or any(n.GetIdx() in flavone_atoms for n in neighbor.GetNeighbors()):
                hydroxy_count += 1
                
    if hydroxy_count == 5:
        return True, "Found flavone with exactly 5 hydroxy groups"
    else:
        return False, f"Found flavone but with {hydroxy_count} hydroxy groups instead of 5"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:25883',
                          'name': 'pentahydroxyflavone',
                          'definition': 'A hydroxyflavone substituted by five '
                                        'hydroxy groups.',
                          'parents': ['CHEBI:24698']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 183901,
    'num_false_negatives': 3,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.9999836871411171}


"""
Classifies: CHEBI:231547 unsaturated fatty aldehyde
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_unsaturated_fatty_aldehyde(smiles: str):
    """
    Determines if a molecule is an unsaturated fatty aldehyde.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is an unsaturated fatty aldehyde, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"
        
    # Check for aldehyde group (C=O)
    has_aldehyde = False
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == 'C':
            for neighbor in atom.GetNeighbors():
                if neighbor.GetSymbol() == 'O' and neighbor.GetDegree() == 1:
                    has_aldehyde = True
                    break
    
    if not has_aldehyde:
        return False, "No aldehyde group found"
        
    # Check for unsaturation (C=C or C#C)
    has_unsaturation = False
    for bond in mol.GetBonds():
        if bond.GetBondType() in [Chem.rdchem.BondType.DOUBLE, Chem.rdchem.BondType.TRIPLE]:
            # Make sure we're not counting the aldehyde C=O
            if not (bond.GetBeginAtom().GetSymbol() == 'O' or bond.GetEndAtom().GetSymbol() == 'O'):
                has_unsaturation = True
                break
                
    if not has_unsaturation:
        return False, "No carbon-carbon multiple bonds found"
        
    # Check carbon chain length (should be fatty - at least 4 carbons)
    carbon_count = sum(1 for atom in mol.GetAtoms() if atom.GetSymbol() == 'C')
    if carbon_count < 4:
        return False, "Carbon chain too short to be considered fatty"
        
    # If we get here, it meets all criteria
    return True, f"Unsaturated fatty aldehyde with {carbon_count} carbons"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:231547',
                          'name': 'unsaturated fatty aldehyde',
                          'definition': 'Any fatty aldehyde containing at '
                                        'least one C=C or C#C bond.',
                          'parents': ['CHEBI:35746']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 19,
    'num_false_positives': 100,
    'num_true_negatives': 165,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.15966386554621848,
    'recall': 1.0,
    'f1': 0.2753623188405797,
    'accuracy': 0.647887323943662}


"""
Classifies: CHEBI:23133 chlorobenzoate
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors

def is_chlorobenzoate(smiles: str):
    """
    Determines if a molecule is a chlorobenzoate.
    Chlorobenzoates are benzoates in which the benzene ring is substituted by at least one chloro group.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a chlorobenzoate, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check for benzoate group
    benzoate_pattern = Chem.MolFromSmarts("[O-]C(=O)c")
    if not mol.HasSubstructMatch(benzoate_pattern):
        return False, "Molecule does not contain a benzoate group"

    # Check for chlorine substituents on the benzene ring
    benzene_ring = Chem.MolFromSmarts("c1ccccc1")
    matches = mol.GetSubstructMatches(benzene_ring)

    if not matches:
        return False, "Molecule does not contain a benzene ring"

    ring_atoms = set(matches[0])
    chloro_substituents = []

    for atom_idx in ring_atoms:
        atom = mol.GetAtomWithIdx(atom_idx)
        for neighbor in atom.GetNeighbors():
            if neighbor.GetIdx() not in ring_atoms and neighbor.GetSymbol() == 'Cl':
                chloro_substituents.append(neighbor.GetSymbol())

    if not chloro_substituents:
        return False, "Benzene ring does not have chloro substituents"

    return True, f"Chlorobenzoate with chloro substituents: {', '.join(chloro_substituents)}"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:23133',
                          'name': 'chlorobenzoate',
                          'definition': 'Any member of the class of benzoates '
                                        'in which the benzene ring is '
                                        'substituted by at least one chloro '
                                        'group.',
                          'parents': ['CHEBI:22718']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 1,
    'num_false_positives': 22,
    'num_true_negatives': 183901,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.043478260869565216,
    'recall': 1.0,
    'f1': 0.08333333333333333,
    'accuracy': 0.9998803853765685}


"""
Classifies: CHEBI:24697 hydroxyflavanone
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_hydroxyflavanone(smiles: str):
    """
    Determines if a molecule is a hydroxyflavanone (flavanone with one or more hydroxy substituents).
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a hydroxyflavanone, False otherwise 
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Flavanone core structure pattern
    # Chroman-4-one fused with phenyl at position 2
    flavanone_pattern = Chem.MolFromSmarts('[#6]1-[#6]-[#6](=O)-c2c(O1)cccc2')
    
    # Check for flavanone core
    if not mol.HasSubstructMatch(flavanone_pattern):
        return False, "Does not contain flavanone core structure"
    
    # Check for hydroxy groups
    hydroxy_pattern = Chem.MolFromSmarts('[OH]')
    hydroxy_matches = mol.GetSubstructMatches(hydroxy_pattern)
    
    if not hydroxy_matches:
        return False, "No hydroxy substituents found"

    # Get core atoms
    core_match = mol.GetSubstructMatch(flavanone_pattern)
    core_atoms = set(core_match)

    # Count hydroxy groups attached to core structure
    core_hydroxy_count = 0
    for match in hydroxy_matches:
        oh_atom = mol.GetAtomWithIdx(match[0])
        for neighbor in oh_atom.GetNeighbors():
            if neighbor.GetIdx() in core_atoms or any(n.GetIdx() in core_atoms for n in neighbor.GetNeighbors()):
                core_hydroxy_count += 1
                break
    
    if core_hydroxy_count == 0:
        return False, "No hydroxy groups attached to flavanone core"
        
    return True, f"Hydroxyflavanone with {core_hydroxy_count} hydroxy group(s) attached to core"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:24697',
                          'name': 'hydroxyflavanone',
                          'definition': 'A member of the class of flavanones '
                                        'that consists of flavanone with one '
                                        'or more hydroxy substituents.',
                          'parents': ['CHEBI:28863', 'CHEBI:33822']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': '\n'
               'Attempt failed: F1 score of 0 is too low.\n'
               'True positives: []\n'
               'False positives: '
               "[('ClC1=C(O)C2=C(OC3C=C(O)C=C(C3C2=O)C(=O)OC)C=C1C', "
               "'Hydroxyflavanone with 2 hydroxy group(s)'), "
               "('O=C1C2=C(OC3C1C(C(=O)OC)=CC(O)=C3)C=C(C)C=C2OC', "
               "'Hydroxyflavanone with 1 hydroxy group(s)'), "
               "('ClC1=C2OC3C=C(O)C=C(C3C(C2=C(O)C=C1C)=O)C(=O)O', "
               "'Hydroxyflavanone with 3 hydroxy group(s)'), "
               "('ClC1=C2OC3C=C(O)C=C(C3C(C2=C(OC)C=C1C)=O)C(=O)OC', "
               "'Hydroxyflavanone with 1 hydroxy group(s)'), "
               "('ClC1=C2OC3C=C(O)C=C(C3C(C2=C(O)C=C1C)=O)C(=O)OC', "
               "'Hydroxyflavanone with 2 hydroxy group(s)'), "
               "('ClC1=C2OC3C=C(O)C=C(C3C(C2=C(O)C(=C1C)Cl)=O)C(=O)O', "
               "'Hydroxyflavanone with 3 hydroxy group(s)')]\n"
               'False negatives: '
               "[('O1C2=C(C(C[C@]1(C3=CC=CC=C3)O)=O)C(=CC(=C2)O)O', 'Does not "
               "contain flavanone core structure'), "
               "('CC(C)=CCCC1(C)Oc2c(O)ccc([C@@H]3CC(=O)c4c(O3)cc(O)c(CC=C(C)C)c4O)c2C=C1', "
               "'Does not contain flavanone core structure'), "
               "('CC(C)=CCc1cc2C(=O)C[C@H](Oc2c(CC=C(C)C)c1O)c1ccc2OC(C)(C)C=Cc2c1', "
               "'Does not contain flavanone core structure'), "
               "('OC[C@H]1O[C@@H](Oc2ccc3C(=O)C[C@H](Oc3c2)c2ccc(O)c(O[C@@H]3O[C@H](CO)[C@@H](O)[C@H](O)[C@H]3O)c2)[C@H](O)[C@@H](O)[C@@H]1O', "
               "'Does not contain flavanone core structure'), "
               "('O[C@@H]1CO[C@H](O[C@H]2[C@H](Oc3cc(O)cc(O)c3C2=O)c2ccc(O)c(O)c2)[C@@H](O)[C@@H]1O', "
               "'Does not contain flavanone core structure'), "
               "('CC(C)=CCOc1ccc(cc1)[C@@H]1CC(=O)c2c(O)cc(O)cc2O1', 'Does not "
               "contain flavanone core structure'), "
               "('Oc1ccc(cc1)[C@@H]1CC(=O)c2c(O)cc(O)cc2O1', 'Does not contain "
               "flavanone core structure'), "
               "('OC(=O)CCC\\\\C=C\\\\C(c1ccccc1)c1c(O)cc2OC(CC(=O)c2c1O)c1ccccc1', "
               "'Does not contain flavanone core structure'), "
               "('OC(=O)CCCC(\\\\C=C\\\\c1ccccc1)c1c(O)cc2OC(CC(=O)c2c1O)c1ccccc1', "
               "'Does not contain flavanone core structure'), "
               "('Oc1ccc2C(=O)CC(Oc2c1O)c1ccccc1', 'Does not contain flavanone "
               "core structure'), ('Oc1cc(O)cc(c1)C1CC(=O)c2c(O)cc(O)cc2O1', "
               "'Does not contain flavanone core structure'), "
               "('COc1ccc(O)c(c1)[C@@H]1CC(=O)c2c(O)c(C)c(O[C@@H]3O[C@H](CO)[C@@H](O)[C@H](O)[C@H]3O)c(C)c2O1', "
               "'Does not contain flavanone core structure'), "
               "('O[C@@H]1CO[C@@H](O[C@H]2[C@@H](Oc3cc(O)cc(O)c3C2=O)c2ccc(O)c(O)c2)[C@H](O)[C@H]1O', "
               "'Does not contain flavanone core structure'), "
               "('O1C2=C(C(C(C1(C3=C(C(=C(C(=C3*)*)*)*)*)O)*)=O)C(=C(C(=C2*)*)*)*', "
               "'Does not contain flavanone core structure'), "
               "('CC(C)=CCC\\\\C(C)=C\\\\Cc1c(O)c(O)ccc1[C@@H]1CC(=O)c2c(O1)cc1OC(C)(C)C(O)Cc1c2O', "
               "'Does not contain flavanone core structure'), "
               "('O[C@@H]1[C@@H](Oc2cc(O)cc(O)c2C1=O)c1ccc(O)c(O)c1', 'Does "
               "not contain flavanone core structure'), "
               "('Oc1cccc(c1)[C@@H]1CC(=O)c2ccccc2O1', 'Does not contain "
               "flavanone core structure'), "
               "('O[C@@H]1[C@H](Oc2c([C@H]3[C@@H](Oc4cc(O)cc(O)c4C3=O)c3ccc(O)c(O)c3)c(O)cc(O)c2C1=O)c1ccc(O)c(O)c1', "
               "'Does not contain flavanone core structure'), "
               "('COc1ccc(cc1O)-c1cc(=O)c2c(O)cc(O[C@@H]3O[C@H](CO[C@@H]4O[C@@H](C)[C@H](O)[C@@H](O)[C@H]4O)[C@@H](O)[C@H](O)[C@H]3O)cc2o1', "
               "'Does not contain flavanone core structure'), "
               "('CC(C)=CCc1c(O)c2OC(C)(C)C=Cc2cc1[C@@H]1CC(=O)c2c(O)cc(O)c(CC=C(C)C)c2O1', "
               "'Does not contain flavanone core structure'), "
               "('COc1cc(ccc1O)[C@@H]1CC(=O)c2c(O)cc(O)cc2O1', 'Does not "
               "contain flavanone core structure'), "
               "('O[C@@H]1CO[C@H](O[C@@H]2[C@H](Oc3cc(O)cc(O)c3C2=O)c2ccc(O)c(O)c2)[C@@H](O)[C@@H]1O', "
               "'Does not contain flavanone core structure')]",
    'attempt': 2,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 19,
    'num_false_positives': 100,
    'num_true_negatives': 23553,
    'num_false_negatives': 3,
    'num_negatives': None,
    'precision': 0.15966386554621848,
    'recall': 0.8636363636363636,
    'f1': 0.2695035460992908,
    'accuracy': 0.995649419218585}


"""
Classifies: CHEBI:25627 octadecadienoic acid
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import rdMolDescriptors

def is_octadecadienoic_acid(smiles: str):
    """
    Determines if a molecule is an octadecadienoic acid (C18 fatty acid with 2 double bonds).
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is an octadecadienoic acid, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Check for carboxylic acid group
    carboxylic_pattern = Chem.MolFromSmarts('[CX3](=O)[OX2H1]')
    if not mol.HasSubstructMatch(carboxylic_pattern):
        return False, "No carboxylic acid group found"

    # Count carbon atoms
    carbon_count = sum(1 for atom in mol.GetAtoms() if atom.GetSymbol() == 'C')
    if carbon_count != 18:
        return False, f"Carbon count is {carbon_count}, should be 18"

    # Count double bonds (excluding the carboxylic acid C=O)
    double_bond_pattern = Chem.MolFromSmarts('C=C')
    double_bond_count = len(mol.GetSubstructMatches(double_bond_pattern))
    
    if double_bond_count != 2:
        return False, f"Number of C=C double bonds is {double_bond_count}, should be 2"

    # Check if it's a straight chain
    # First create a copy without explicit H atoms
    mol_no_h = Chem.RemoveHs(mol)
    
    # Count rings
    if rdMolDescriptors.CalcNumRings(mol_no_h) > 0:
        return False, "Contains rings - not a straight chain"

    # Check for branching by counting number of non-H neighbors for each carbon
    for atom in mol_no_h.GetAtoms():
        if atom.GetSymbol() == 'C':
            non_h_neighbors = len([n for n in atom.GetNeighbors() 
                                 if n.GetSymbol() != 'H'])
            if non_h_neighbors > 4:  # No carbon should have more than 4 non-H neighbors
                return False, "Contains branching - not a straight chain"
            if atom.GetIsAromatic():
                return False, "Contains aromatic carbons"

    return True, "Valid octadecadienoic acid"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:25627',
                          'name': 'octadecadienoic acid',
                          'definition': 'Any straight-chain, C18 '
                                        'polyunsaturated fatty acid having two '
                                        'C=C double bonds.',
                          'parents': [   'CHEBI:140949',
                                         'CHEBI:15904',
                                         'CHEBI:26208',
                                         'CHEBI:36326',
                                         'CHEBI:53339',
                                         'CHEBI:59202']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': "Attempt failed: module 'rdkit.Chem.rdMolDescriptors' has no "
               "attribute 'CalcNumAliphaticDoubleBonds'",
    'attempt': 1,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 6,
    'num_false_positives': 100,
    'num_true_negatives': 141641,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.05660377358490566,
    'recall': 1.0,
    'f1': 0.10714285714285715,
    'accuracy': 0.9992945176970236}


"""
Classifies: CHEBI:22801 beta-D-glucosyl-N-acylsphingosine
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_beta_D_glucosyl_N_acylsphingosine(smiles: str):
    """
    Determines if a molecule is a beta-D-glucosyl-N-acylsphingosine.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a beta-D-glucosyl-N-acylsphingosine, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check for the presence of a glucosyl head group
    glucosyl_head = Chem.MolFromSmiles('O[C@@H]1[C@H](O)[C@H](O)[C@@H](O)[C@H](O)[C@H]1O')
    if not mol.HasSubstructMatch(glucosyl_head):
        return False, "No glucosyl head group found"

    # Check for beta configuration at the anomeric center
    anomeric_atom = mol.GetAtomWithIdx(0)
    if anomeric_atom.GetHybridization() != Chem.HybridizationType.SP3 or \
       anomeric_atom.GetTotalNumHs() != 0:
        return False, "Anomeric center is not in beta configuration"

    # Check for the presence of an acyl group
    acyl_group = Chem.MolFromSmiles('CC(=O)')
    if not mol.HasSubstructMatch(acyl_group):
        return False, "No acyl group found"

    # Check for the presence of a sphingosine chain
    sphingosine_chain = Chem.MolFromSmiles('C=CCCCCCCCCCCCC')
    if not mol.HasSubstructMatch(sphingosine_chain):
        return False, "No sphingosine chain found"

    return True, "Molecule is a beta-D-glucosyl-N-acylsphingosine"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:22801',
                          'name': 'beta-D-glucosyl-N-acylsphingosine',
                          'definition': 'A D-glucosyl-N-acylsphingosine in '
                                        'which the glucosyl head group has '
                                        'beta-configuration at the anomeric '
                                        'centre.',
                          'parents': ['CHEBI:18368', 'CHEBI:83264']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 45,
    'num_true_negatives': 183878,
    'num_false_negatives': 1,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0,
    'accuracy': 0.9997498966964616}


"""
Classifies: CHEBI:166986 alkyldiacylglyceride
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import rdMolDescriptors

def is_alkyldiacylglyceride(smiles: str):
    """
    Determines if a molecule is an alkyldiacylglyceride - a triradylglycerol with 
    a single alkyl substituent and two acyl substituents.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is an alkyldiacylglyceride, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check for glycerol backbone
    glycerol_pattern = Chem.MolFromSmarts("[CH2]-[CH]-[CH2]")
    if not mol.HasSubstructMatch(glycerol_pattern):
        return False, "No glycerol backbone found"

    # Count O-substituents
    o_substituents = 0
    acyl_groups = 0
    alkyl_groups = 0

    # Patterns for substituents
    acyl_pattern = Chem.MolFromSmarts("O=C-O")  # For acyl groups
    alkyl_pattern = Chem.MolFromSmarts("CO")     # For alkyl groups (C-O single bond)

    # Find acyl groups
    acyl_matches = mol.GetSubstructMatches(acyl_pattern)
    acyl_groups = len(acyl_matches)

    # Find total O-substituents
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == 'O':
            o_substituents += 1

    # Calculate alkyl groups (O atoms not part of acyl groups)
    alkyl_groups = o_substituents - (2 * acyl_groups)  # Each acyl group has 2 oxygens

    if acyl_groups == 2 and alkyl_groups == 1:
        return True, f"Alkyldiacylglyceride with {acyl_groups} acyl groups and {alkyl_groups} alkyl group"
    elif acyl_groups != 2:
        return False, f"Found {acyl_groups} acyl groups, requires exactly 2"
    elif alkyl_groups != 1:
        return False, f"Found {alkyl_groups} alkyl groups, requires exactly 1"
    else:
        return False, "Does not match alkyldiacylglyceride pattern"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:166986',
                          'name': 'alkyldiacylglyceride',
                          'definition': 'A triradylglycerol with a single '
                                        'alkyl substituent and two acyl '
                                        'substituents at unspecified '
                                        'positions.',
                          'parents': [   'CHEBI:24353',
                                         'CHEBI:33308',
                                         'CHEBI:47778',
                                         'CHEBI:64611',
                                         'CHEBI:76579']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 1,
    'num_false_positives': 100,
    'num_true_negatives': 16717,
    'num_false_negatives': 1,
    'num_negatives': None,
    'precision': 0.009900990099009901,
    'recall': 0.5,
    'f1': 0.019417475728155338,
    'accuracy': 0.99399488673524}


"""
Classifies: CHEBI:22720 benzodiazepine
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_benzodiazepine(smiles: str):
    """
    Determines if a molecule is a benzodiazepine (contains a benzene ring fused to a diazepine ring).
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a benzodiazepine, False otherwise
        str: Reason for classification
    """
    # Generate molecule from SMILES
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Generate ring information
    rings = mol.GetRingInfo()
    
    # Find all rings of size 6 (benzene) and 7 (diazepine)
    six_membered_rings = []
    seven_membered_rings = []
    
    for ring in rings.AtomRings():
        if len(ring) == 6:
            six_membered_rings.append(ring)
        elif len(ring) == 7:
            seven_membered_rings.append(ring)
            
    if not six_membered_rings:
        return False, "No 6-membered rings found"
    if not seven_membered_rings:
        return False, "No 7-membered rings found"
    
    # Check for benzene rings (6-membered aromatic rings with all carbons)
    benzene_rings = []
    for ring in six_membered_rings:
        atoms = [mol.GetAtomWithIdx(i) for i in ring]
        if all(atom.GetIsAromatic() for atom in atoms) and \
           all(atom.GetSymbol() == 'C' for atom in atoms):
            benzene_rings.append(ring)
            
    if not benzene_rings:
        return False, "No benzene rings found"
        
    # Check for diazepine rings (7-membered rings with exactly 2 nitrogens)
    diazepine_rings = []
    for ring in seven_membered_rings:
        atoms = [mol.GetAtomWithIdx(i) for i in ring]
        n_count = sum(1 for atom in atoms if atom.GetSymbol() == 'N')
        if n_count == 2:
            diazepine_rings.append(ring)
            
    if not diazepine_rings:
        return False, "No diazepine rings found"
    
    # Check if benzene and diazepine rings are fused
    for benz_ring in benzene_rings:
        for diaz_ring in diazepine_rings:
            # Two rings are fused if they share exactly 2 adjacent atoms
            shared_atoms = set(benz_ring).intersection(set(diaz_ring))
            if len(shared_atoms) == 2:
                # Check if shared atoms are adjacent
                shared_list = list(shared_atoms)
                bond = mol.GetBondBetweenAtoms(shared_list[0], shared_list[1])
                if bond is not None:
                    return True, "Contains benzene ring fused to diazepine ring"
                
    return False, "Benzene and diazepine rings are not fused"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:22720',
                          'name': 'benzodiazepine',
                          'definition': 'A group of heterocyclic compounds '
                                        'with a core structure containing a '
                                        'benzene ring fused to a diazepine '
                                        'ring.',
                          'parents': ['CHEBI:38101', 'CHEBI:38166']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': "Attempt failed: cannot import name 'rdDecomposition' from "
               "'rdkit.Chem' "
               '(/Users/cjm/Library/Caches/pypoetry/virtualenvs/c3p-93U7KWO_-py3.11/lib/python3.11/site-packages/rdkit/Chem/__init__.py)',
    'attempt': 1,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 17,
    'num_false_positives': 100,
    'num_true_negatives': 95670,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.1452991452991453,
    'recall': 1.0,
    'f1': 0.2537313432835821,
    'accuracy': 0.9989560169960433}


"""
Classifies: CHEBI:15904 long-chain fatty acid
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem.AllChem import FindMolChiralCenters
from collections import deque

def find_longest_carbon_chain(mol):
    """Find the longest continuous carbon chain containing the carboxylic acid group."""
    # Find carboxylic acid carbon
    carboxylic_pattern = Chem.MolFromSmarts('C(=O)[OH]')
    if not mol.HasSubstructMatch(carboxylic_pattern):
        return []
    
    matches = mol.GetSubstructMatches(carboxylic_pattern)
    carboxylic_carbon = matches[0][0]
    
    # BFS to find longest carbon chain
    visited = set()
    queue = deque([(carboxylic_carbon, [carboxylic_carbon])])
    longest_path = []
    
    while queue:
        current, path = queue.popleft()
        atom = mol.GetAtomWithIdx(current)
        
        for neighbor in atom.GetNeighbors():
            neighbor_idx = neighbor.GetIdx()
            if neighbor_idx not in visited and neighbor.GetSymbol() == 'C':
                new_path = path + [neighbor_idx]
                queue.append((neighbor_idx, new_path))
                if len(new_path) > len(longest_path):
                    longest_path = new_path
                visited.add(neighbor_idx)
    
    return longest_path

def is_long_chain_fatty_acid(smiles: str):
    """
    Determines if a molecule is a long-chain fatty acid (C13-C22).
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a long-chain fatty acid, False otherwise
        str: Reason for classification
    """
    # Create RDKit mol object
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"
    
    # Check for carboxylic acid group
    carboxylic_pattern = Chem.MolFromSmarts('C(=O)[OH]')
    if not mol.HasSubstructMatch(carboxylic_pattern):
        return False, "No carboxylic acid group found"
    
    # Find longest carbon chain containing carboxylic acid group
    longest_chain = find_longest_carbon_chain(mol)
    chain_length = len(longest_chain)
    
    if chain_length < 13:
        return False, f"Carbon chain too short (C{chain_length})"
    elif chain_length > 22:
        return False, f"Carbon chain too long (C{chain_length})"
    
    # Check if the carbon chain is the main feature (not part of a larger aromatic/complex system)
    aromatic_pattern = Chem.MolFromSmarts('c1ccccc1')
    complex_ring_pattern = Chem.MolFromSmarts('[R]')
    
    # Get atoms in rings
    ring_atoms = set()
    for ring in mol.GetRingInfo().AtomRings():
        ring_atoms.update(ring)
    
    # Count carbons in longest chain that are part of rings
    chain_ring_atoms = sum(1 for atom_idx in longest_chain if atom_idx in ring_atoms)
    
    # If more than 1/3 of the chain is in rings, it's likely not a fatty acid
    if chain_ring_atoms > len(longest_chain) / 3:
        return False, "Carbon chain is part of a complex ring system"
    
    # Check for peptide bonds which would indicate it's not a fatty acid
    peptide_pattern = Chem.MolFromSmarts('[NX3][CX3](=[OX1])[CX4]')
    if mol.HasSubstructMatch(peptide_pattern):
        return False, "Molecule contains peptide bonds"
    
    return True, f"Long-chain fatty acid with {chain_length} carbons"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:15904',
                          'name': 'long-chain fatty acid',
                          'definition': 'A fatty acid with a chain length '
                                        'ranging from C13 to C22.',
                          'parents': ['CHEBI:35366']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': '\n'
               'Attempt failed: F1 score of 0.7685774946921445 is too low.\n'
               "True positives: [('OC(=O)CCCC#CCCCCCCC', 'Long-chain fatty "
               "acid with 13 carbons'), ('CCCCCCCCCCCCCCCC(O)CC(O)=O', "
               "'Long-chain fatty acid with 18 carbons'), "
               "('CC\\\\C=C/C\\\\C=C/C=C/C(O)CCCCCCCC(O)=O', 'Long-chain fatty "
               "acid with 18 carbons'), "
               "('OC(=O)CC/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\CCCCCC', "
               "'Long-chain fatty acid with 20 carbons'), "
               "('O=C(CCCCCCCCC)CCCCCC(O)=O', 'Long-chain fatty acid with 16 "
               "carbons'), "
               "('CCCCC[C@@H]1O[C@H]1[C@H](O)\\\\C=C/C\\\\C=C/C\\\\C=C/CCCC(O)=O', "
               "'Long-chain fatty acid with 20 carbons'), "
               "('O[C@@H](CCCCCCCCCCC)CCCC(O)=O', 'Long-chain fatty acid with "
               "16 carbons'), ('O(C(CCCCCCCCCC(O)=O)CCCCCCC)C', 'Long-chain "
               "fatty acid with 19 carbons'), "
               "('O=C(CCCCC)/C=C\\\\C=C\\\\C\\\\C=C\\\\CCCC(O)=O', 'Long-chain "
               "fatty acid with 17 carbons'), ('OC(=O)CCCCCCCCCCCCCCCC(CC)C', "
               "'Long-chain fatty acid with 20 carbons'), "
               "('O1[C@@H]([C@H]([C@@H](O)CC1=O)C/C=C\\\\CCCC(O)=O)/C=C/[C@@H](O)C/C=C\\\\CC', "
               "'Long-chain fatty acid with 20 carbons'), "
               "('CC/C=C\\\\C/C=C\\\\CC(C(CCCCCCCC(O)=O)O)O', 'Long-chain "
               "fatty acid with 18 carbons'), "
               "('OC(=O)CCCCCCCCCCCCCCC.OC(CO)CO', 'Long-chain fatty acid with "
               "19 carbons'), "
               "('O[C@@H](CCCCCC/C=C/[C@H](O)[C@@H](O)[C@H](O)[C@H](N=C(N)N)C(O)=O)CCCCCC', "
               "'Long-chain fatty acid with 21 carbons'), "
               "('O[C@@H](CCCC(O)=O)\\\\C=C/C=C/C=C/C(=O)C/C=C\\\\C=C\\\\[C@@H](O)CC', "
               "'Long-chain fatty acid with 20 carbons'), "
               "('[H]C(\\\\C=C/CCCCCCCC(O)=O)=C1O[C@H]1C\\\\C=C/CC', "
               "'Long-chain fatty acid with 18 carbons'), "
               "('O=C(O)CCCCCCCC/C=C\\\\CC/C=C\\\\CCCCC', 'Long-chain fatty "
               "acid with 20 carbons'), ('OC(CCCCCCCCCC)C(O)CCCCCC(O)=O', "
               "'Long-chain fatty acid with 18 carbons'), "
               "('O=C(CCCCCCCC(O)=O)CCCCCCC', 'Long-chain fatty acid with 16 "
               "carbons'), ('O[C@H](CCCCCCCCCC(O)=O)/C=C/CCCCC', 'Long-chain "
               "fatty acid with 18 carbons'), "
               "('CCCC\\\\C=C\\\\C=C\\\\C=C/CCCCCCCC(O)=O', 'Long-chain fatty "
               "acid with 18 carbons'), "
               "('OC(=O)[C@H](CCCCCCCCCCCCCCCC)C(C(O)=O)=C', 'Long-chain fatty "
               "acid with 21 carbons'), ('CCCCCCCCCCC[C@H](N)C(O)=O', "
               "'Long-chain fatty acid with 13 carbons'), "
               "('OC(=O)CCCCCCCCCCCCCCCCCC(CC)C', 'Long-chain fatty acid with "
               "22 carbons'), ('OC(CCCCCCC/C=C\\\\CCCCCC)CC(O)=O', 'Long-chain "
               "fatty acid with 18 carbons'), ('O=C(CCCCCCCCC)CCCCCCCC(O)=O', "
               "'Long-chain fatty acid with 18 carbons'), "
               "('OC(CCCCCCCCCC)CC(O)=O', 'Long-chain fatty acid with 13 "
               "carbons'), "
               "('C(\\\\C=C\\\\C=C\\\\C([C@@H](O)C/C=C\\\\C/C=C\\\\CC)O)=C\\\\CCCCCC(O)=O', "
               "'Long-chain fatty acid with 22 carbons'), "
               "('OC(CCCC(O)=O)C=CC=CC=CC(O)CC=CCCC(O)=O', 'Long-chain fatty "
               "acid with 18 carbons'), "
               "('CCCCC\\\\C=C/C[C@@H](O)\\\\C=C\\\\C=C/C\\\\C=C/CCCC(O)=O', "
               "'Long-chain fatty acid with 20 carbons'), "
               "('CCCCCCCCC\\\\C=C/CCCCC(O)=O', 'Long-chain fatty acid with 16 "
               "carbons'), "
               "('C(C(O)=O)C/C=C\\\\C/C=C\\\\C\\\\C=C/C=C/[C@@H](C/C=C\\\\C/C=C\\\\CC)O', "
               "'Long-chain fatty acid with 22 carbons'), "
               "('O=C(CCCCCCCCCCCCCCC(O)=O)C', 'Long-chain fatty acid with 17 "
               "carbons'), ('OC(=O)CCCCCCCC#CC#CC#C/C=C/C=C', 'Long-chain "
               "fatty acid with 18 carbons'), ('OCCCCCCCCC/C=C/CCCCC(O)=O', "
               "'Long-chain fatty acid with 16 carbons'), "
               "('OCCCCCCCCCCCCCCC\\\\C=C\\\\C(O)=O', 'Long-chain fatty acid "
               "with 18 carbons'), ('CCCCC\\\\C=C/C\\\\C=C/CCCC(O)=O', "
               "'Long-chain fatty acid with 14 carbons'), "
               "('OC(CC/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\C=C\\\\1/[C@H](C/C=C\\\\CC)O1)=O', "
               "'Long-chain fatty acid with 22 carbons'), "
               "('OC(=O)C(CCCC(CCCCCCCCC)C)C', 'Long-chain fatty acid with 17 "
               "carbons'), ('OC(=O)CCCCCCCCCC/C=C\\\\C/C=C\\\\CC', 'Long-chain "
               "fatty acid with 18 carbons'), "
               "('CCCCCCCC\\\\C(=C/CCCCCCCC(O)=O)[N+]([O-])=O', 'Long-chain "
               "fatty acid with 18 carbons'), "
               "('O1[C@](C[C@@H](O)[C@H]1CC)([C@@H](O)/C=C/[C@@H](O)CCCCCCCC(O)=O)[H]', "
               "'Long-chain fatty acid with 18 carbons'), "
               "('C(O)(=O)CC/C=C\\\\C[C@H]1[C@H](/C=C/C=C/C=C\\\\C=C\\\\[C@@H](C/C=C\\\\CC)O)O1', "
               "'Long-chain fatty acid with 22 carbons'), "
               "('O1C(C1C(OO)/C=C/CCCCCCCC(O)=O)CCCCC', 'Long-chain fatty acid "
               "with 18 carbons'), ('O=C(O)[C@H](CC(=O)CCCCCCCCCCCCC)C', "
               "'Long-chain fatty acid with 18 carbons'), "
               "('C(CCCCCCCCCC)CCCCCCC(=O)O', 'Long-chain fatty acid with 18 "
               "carbons'), ('O[C@H](CCCCCC)\\\\C=C\\\\CCCCCCCCC(O)=O', "
               "'Long-chain fatty acid with 18 carbons'), "
               "('OCCCCCCCCCC\\\\C=C\\\\C(O)=O', 'Long-chain fatty acid with "
               "13 carbons'), "
               "('O=C(CCC/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\[C@H](CCCC)O)O', "
               "'Long-chain fatty acid with 20 carbons'), "
               "('OC(=O)C\\\\C=C\\\\C=C/C=C=CC#CC#C', 'Long-chain fatty acid "
               "with 13 carbons'), "
               "('OC(=O)CCC/C=C\\\\C(/C=C\\\\CCCCCCCCCCC)(C)C', 'Long-chain "
               "fatty acid with 22 carbons'), ('O[C@H](CCCCCCCCCC)CC(O)=O', "
               "'Long-chain fatty acid with 13 carbons'), "
               "('O[C@@H](CCCCCCCC(O)=O)/C=C/[C@H](O)[C@@H](O)C/C=C\\\\CC', "
               "'Long-chain fatty acid with 18 carbons'), "
               "('C(\\\\CC)=C\\\\C/C=C\\\\CC(/C=C/C=C\\\\C/C=C\\\\CCCC(=O)O)O', "
               "'Long-chain fatty acid with 20 carbons'), "
               "('OC(=O)CCC(CCCC(CCCCCCC)C)C', 'Long-chain fatty acid with 17 "
               "carbons'), "
               "('O[C@H](CCCCCCCC(O)=O)\\\\C=C\\\\[C@@H]1[C@H](CC)C(=O)C=C1', "
               "'Long-chain fatty acid with 18 carbons'), "
               "('CCCCCCCC[C@H](OO)\\\\C=C\\\\CCCCCCC(O)=O', 'Long-chain fatty "
               "acid with 18 carbons'), ('OC(CC(O)CCC(O)C)CCCCCCCCC(O)=O', "
               "'Long-chain fatty acid with 16 carbons'), "
               "('OC(=O)CCCCCCCCCCCC(C)(C)C', 'Long-chain fatty acid with 16 "
               "carbons'), ('O[13C](=O)[13CH2][13CH2][13CH2]CCCCCCCCCCCC', "
               "'Long-chain fatty acid with 16 carbons'), "
               "('OC(=O)CCCCCC#CCC#CCCCCCC', 'Long-chain fatty acid with 17 "
               "carbons'), "
               "('C(C(O)=O)C/C=C\\\\C/C=C\\\\C\\\\C=C/C=C/C(C/C=C\\\\C/C=C\\\\CC)O', "
               "'Long-chain fatty acid with 22 carbons'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/CCCC\\\\C=C/CCCC(O)=O', "
               "'Long-chain fatty acid with 20 carbons'), "
               "('OC(=O)CCCC/C=C/C=C/C#CC#C\\\\C=C\\\\C', 'Long-chain fatty "
               "acid with 16 carbons'), ('CCCCCCC#CCCCCCCCCCC(O)=O', "
               "'Long-chain fatty acid with 18 carbons'), "
               "('CCCCCC(=O)CC\\\\C=C/C=C/C=C/[C@@H](O)[C@@H](O)CCCC(O)=O', "
               "'Long-chain fatty acid with 20 carbons'), "
               "('O=C1C(C(=O)N)=CNC(=C1)/C=C\\\\CCCCCCCCC(=O)O', 'Long-chain "
               "fatty acid with 17 carbons'), ('C(CCCCCCC(C)O)CCCCC(O)=O', "
               "'Long-chain fatty acid with 14 carbons'), "
               "('OC(=O)CCCCCC#CCCCCCCCCCC', 'Long-chain fatty acid with 18 "
               "carbons'), "
               "('[H]C(CCCC(O)=O)=CCC([H])=CCC([H])=CC([H])=CCCCCCC', "
               "'Long-chain fatty acid with 20 carbons'), "
               "('CCCCCCC\\\\C=C\\\\C(CCCCCCCC(O)=O)OO', 'Long-chain fatty "
               "acid with 18 carbons'), ('CCCCCCCCCCCCCCC\\\\C=C\\\\C(O)=O', "
               "'Long-chain fatty acid with 18 carbons'), "
               "('C([C@@H](CC)O)C/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\CCCC(O)=O', "
               "'Long-chain fatty acid with 20 carbons'), "
               "('OCCCCCCCCCCCCCCCC[C@@H](O)CC(O)=O', 'Long-chain fatty acid "
               "with 19 carbons'), "
               "('O(/C(/CCCCCCC)=C\\\\C=C\\\\C=C\\\\C(O)=O)C(=O)C', "
               "'Long-chain fatty acid with 16 carbons'), "
               "('C(CCC/C=C\\\\C/C=C\\\\CC(/C=C/C=C\\\\CCCCC)=O)(=O)O', "
               "'Long-chain fatty acid with 20 carbons'), "
               "('OC(=O)CCCCCCCCCCCCCCCCCC(O)=O', 'Long-chain fatty acid with "
               "19 carbons'), ('OC(=O)CCC(CCCCCCCCCCCCCC)C', 'Long-chain fatty "
               "acid with 19 carbons'), "
               "('[H]C(CCCC(O)=O)=CCC([H])=CCC([H])=CCC([H])=CCCCCC', "
               "'Long-chain fatty acid with 20 carbons'), "
               "('C(C(/C=C/C=C/C=C\\\\[C@H](CCCC(O)=O)O)=O)/C=C\\\\CCCCCO', "
               "'Long-chain fatty acid with 20 carbons'), "
               "('OC(CCCCC(O)=O)/C=C/C=C/C#CC#CC#CC#C', 'Long-chain fatty acid "
               "with 18 carbons'), ('OC(=O)CC(CC(CCCCCCCC)C)C', 'Long-chain "
               "fatty acid with 15 carbons'), "
               "('CCCCCCCC[C@@H](O)[C@@H](O)CCCCCCCC(O)=O', 'Long-chain fatty "
               "acid with 18 carbons'), "
               "('O[C@H]1[C@@H]([C@@H](C(=O)C1)CC)/C=C/[C@@H](O)C/C=C\\\\C/C=C\\\\C/C=C\\\\CCC(O)=O', "
               "'Long-chain fatty acid with 22 carbons'), "
               "('C(\\\\C=C/CCCCCCCC(=O)O)=C/C(C/C=C\\\\CC)OO', 'Long-chain "
               "fatty acid with 18 carbons'), "
               "('[H]C(CCCCCCCC(O)=O)=CC([H])=CC([H])=CCCCC', 'Long-chain "
               "fatty acid with 18 carbons'), "
               "('OC(=O)\\\\C=C\\\\CC(CCCCCCCC)C', 'Long-chain fatty acid with "
               "14 carbons'), ('C(CCCCCCC/C=C\\\\C=C\\\\C(CCCCC)O)(=O)O', "
               "'Long-chain fatty acid with 18 carbons'), "
               "('OC(=O)CCC/C=C\\\\CCCCCCCC/C=C\\\\CCCC', 'Long-chain fatty "
               "acid with 20 carbons'), "
               "('C(=C/[C@@H](C\\\\C=C/C=C/[C@H](C/C=C\\\\CC)O)O)\\\\C=C\\\\C=C/[C@H](CCC(O)=O)O', "
               "'Long-chain fatty acid with 22 carbons'), "
               "('OC(=O)CC(CCCCCCCCCCCCC)C', 'Long-chain fatty acid with 17 "
               "carbons'), ('OC(CCCCCCCCC(O)=O)CCCCCC(O)=O', 'Long-chain fatty "
               "acid with 16 carbons'), ('OC(=O)CCCCCCCCCC#CCC', 'Long-chain "
               "fatty acid with 14 carbons'), ('O[C@H](CCCCCCCCCC(O)=O)CCC', "
               "'Long-chain fatty acid with 14 carbons'), "
               "('O=C(CCCCCCCCCCCCCCCCC(O)=O)C', 'Long-chain fatty acid with "
               "19 carbons'), ('OC(=O)CCC/C=C/C\\\\C=C\\\\C\\\\C=C\\\\CCCCCC', "
               "'Long-chain fatty acid with 18 carbons'), "
               "('OC(CCCCCCC)/C=C/CCCCCCCC(O)=O', 'Long-chain fatty acid with "
               "18 carbons'), ('OCCCCCCCCCCC\\\\C=C\\\\C(O)=O', 'Long-chain "
               "fatty acid with 14 carbons'), ('OC(=O)C(C(CCCCCCCCCC)C)(C)C', "
               "'Long-chain fatty acid with 16 carbons'), "
               "('ClC(CCCCCCCC)C(Cl)CCCC(O)=O', 'Long-chain fatty acid with 14 "
               "carbons')]\n"
               'False positives: '
               "[('O=C(N1[C@H](CCC1)C(O)=O)[C@@H](NC(=O)[C@@H](N)[C@H](O)C)CC2=CC=CC=C2', "
               "'Long-chain fatty acid with 18 carbons'), "
               "('O=C1C(OC(=C1/C=C/C(=O)O)CCC)=C(C)C', 'Long-chain fatty acid "
               "with 13 carbons'), "
               "('CS(=O)(=O)C1=CC=C(C=C1)CN2C3=C(CCCC3CC(=O)O)C4=CC(=CC(=C42)F)F', "
               "'Long-chain fatty acid with 22 carbons'), "
               "('O=C(N[C@@H](CC1=CC=CC=C1)C(O)=O)[C@@H](NC(=O)[C@@H](N)CC(=O)N)C', "
               "'Long-chain fatty acid with 16 carbons'), "
               "('OC(=O)C(F)(F)F.[H][C@]12C[C@H](O)CC[C@@]11CCN2Cc2cc(O)c(OC)cc12', "
               "'Long-chain fatty acid with 18 carbons'), "
               "('O[C@H]([C@H](OC(=O)\\\\C=C/c1ccc(O)c(O)c1)C(O)=O)C(O)=O', "
               "'Long-chain fatty acid with 13 carbons'), "
               "('C=1(C=CC(=CC1)[N+]([O-])=O)COP(CCCCCC(O)=O)(=O)O', "
               "'Long-chain fatty acid with 13 carbons'), "
               "('O=C(N1[C@H](CCC1)C(O)=O)[C@@H](NC(=O)CN)CC2=CC=C(O)C=C2', "
               "'Long-chain fatty acid with 16 carbons'), "
               "('O=C(N[C@@H](CC(=O)N)C(O)=O)[C@@H](NC(=O)[C@@H](N)CCC(=O)N)CCCCN', "
               "'Long-chain fatty acid with 15 carbons'), "
               "('O1C(CCCCCCCC(O)=O)=C(C(=C1CCC)C)C', 'Long-chain fatty acid "
               "with 17 carbons'), "
               "('O=C1C=C2[C@@H]3[C@](OC)(CCC[C@H]3/C=C(/C=C(\\\\OC)/C=C/C(=O)O)\\\\C)O[C@H]2[C@@H]4[C@H]1O4', "
               "'Long-chain fatty acid with 22 carbons'), "
               "('[H][C@@]12CCC(=C)[C@H](C[C@H](O)[C@](C)(O)C=C)[C@@]1(C)CCC[C@]2(C)C(O)=O', "
               "'Long-chain fatty acid with 20 carbons'), "
               "('OC(=O)[C@@H](NC(=O)[C@@H](NC(=O)[C@@H](N)CO)CC(O)=O)[C@H](CC)C', "
               "'Long-chain fatty acid with 13 carbons'), "
               "('O=C(N[C@@H](CCC(O)=O)C(=O)N[C@@H]([C@H](O)C)C(O)=O)[C@@H](N)C(C)C', "
               "'Long-chain fatty acid with 14 carbons'), "
               "('CC(C)C[C@H](NC(=O)[C@@H](N)CC(O)=O)C(=O)N[C@@H](CC(C)C)C(=O)N[C@@H](CO)C(O)=O', "
               "'Long-chain fatty acid with 19 carbons'), "
               "('O=C(N[C@@H](CC=1NC=NC1)C(O)=O)[C@@H](NC(=O)[C@@H](N)CC(O)=O)CCC(=O)N', "
               "'Long-chain fatty acid with 15 carbons'), "
               "('O=C(N[C@@H](CC(C)C)C(O)=O)[C@@H](NC(=O)[C@@H](N)CC1=CC=CC=C1)CC(C)C', "
               "'Long-chain fatty acid with 21 carbons'), "
               "('S(CC[C@H](N)C(=O)N[C@H](C(=O)N[C@@H](CC1=CC=C(O)C=C1)C(O)=O)CO)C', "
               "'Long-chain fatty acid with 17 carbons'), "
               "('O=C(N1[C@H](CCC1)C(O)=O)[C@@H](NC(=O)CN)CC2=CC=CC=C2', "
               "'Long-chain fatty acid with 16 carbons'), "
               "('OC(=O)C(O)=O.CCCCCCCCCOc1ccc2[nH]cc(CCN)c2c1', 'Long-chain "
               "fatty acid with 21 carbons'), "
               "('O=C(N[C@@H](CC=1NC=NC1)C(O)=O)[C@@H](NC(=O)[C@@H](N)CCCCN)CCCCN', "
               "'Long-chain fatty acid with 18 carbons'), "
               "('S(CC[C@H](NC(=O)[C@@H](N)CC=1NC=NC1)C(=O)N[C@@H](CC=2C=3C(NC2)=CC=CC3)C(O)=O)C', "
               "'Long-chain fatty acid with 22 carbons'), "
               "('SC[C@H](NC(=O)[C@@H](N)CC1=CC=C(O)C=C1)C(=O)NCC(O)=O', "
               "'Long-chain fatty acid with 14 carbons'), "
               "('O=C(N[C@@H](CCCCN)C(O)=O)[C@@H](NC(=O)[C@@H](N)CO)CC(C)C', "
               "'Long-chain fatty acid with 15 carbons'), "
               "('O=C(N[C@@H](CC1=CC=C(O)C=C1)C(O)=O)[C@@H](NC(=O)[C@@H](N)CCC(O)=O)[C@H](CC)C', "
               "'Long-chain fatty acid with 20 carbons'), "
               "('OC1C(C2C(C3C4(C2C(O)=O)CC(CC3)C(C4)=C)(CC1O)C(O)=O)(C)C(O)=O', "
               "'Long-chain fatty acid with 20 carbons'), "
               "('O=C(N[C@@H](CC(O)=O)C(O)=O)[C@H]1N(CCC1)C(=O)[C@@H](N)CC(O)=O', "
               "'Long-chain fatty acid with 13 carbons'), "
               "('O=C(N[C@@H](C)C(O)=O)[C@@H](NC(=O)[C@@H](N)CC=1NC=NC1)C(C)C', "
               "'Long-chain fatty acid with 14 carbons'), "
               "('O1[C@@H](OC2=CC=C(C3OC=4C(C(=O)C3)=C(O)C=C(O)C4)C=C2)[C@H](O)[C@@H](O)[C@H](O)[C@H]1C(O)=O', "
               "'Long-chain fatty acid with 21 carbons'), "
               "('O=C(N[C@@H](CC1=CC=CC=C1)C(O)=O)[C@@H](NC(=O)CN)CCC(O)=O', "
               "'Long-chain fatty acid with 16 carbons'), "
               "('O=C(N[C@@H]([C@H](CC)C)C(O)=O)[C@@H](NC(=O)[C@@H](N)CC(=O)N)CC(C)C', "
               "'Long-chain fatty acid with 16 carbons'), "
               "('C1CC(CN(C1)CCC=C(C2=CC=CC=C2)C3=CC=CC=C3)C(=O)O', "
               "'Long-chain fatty acid with 22 carbons'), "
               "('O=C(NCC(O)=O)CCC=1C=2C(NC1)=CC=CC2', 'Long-chain fatty acid "
               "with 13 carbons'), "
               "('O=C(O)[C@H]1N(C(=O)[C@@H](NC(=O)[C@@H](NC(=O)[C@@H](N)CCC(=O)N)CC(C)C)C(C)C)CCC1', "
               "'Long-chain fatty acid with 21 carbons'), "
               "('O(CC(COCCCCCC)O)C1=C(C(O)=O)C=CC=C1', 'Long-chain fatty acid "
               "with 16 carbons'), "
               "('O=C1OC2=C(C(=C(C(=O)O)C(=C2COC)O)C)OC3=C1C(=CC(=C3C=O)O)C', "
               "'Long-chain fatty acid with 19 carbons'), "
               "('SC[C@H](NC(=O)[C@@H](N)C)C(=O)N[C@@H](CC=1C=2C(NC1)=CC=CC2)C(O)=O', "
               "'Long-chain fatty acid with 17 carbons'), "
               "('S(CC[C@H](NC(=O)[C@@H](N)CCSC)C(=O)N[C@@H](C(C)C)C(O)=O)C', "
               "'Long-chain fatty acid with 15 carbons'), "
               "('O[C@@H]([C@H](NC(=O)[C@@H](N)CC=1C=2C(NC1)=CC=CC2)C(=O)N[C@@H](CC(=O)N)C(O)=O)C', "
               "'Long-chain fatty acid with 19 carbons'), "
               "('S(CC[C@H](NC(=O)[C@@H](NC(=O)[C@@H](N)CC1=CC=C(O)C=C1)CC(O)=O)C(O)=O)C', "
               "'Long-chain fatty acid with 18 carbons'), "
               "('N[C@@H](CCC(=O)N[C@@H](CSCC(=O)SC[C@H](NC(=O)CC[C@H](N)C(O)=O)C(=O)NCC(O)=O)C(=O)NCC(O)=O)C(O)=O', "
               "'Long-chain fatty acid with 22 carbons'), "
               "('O=C(N1[C@H](CCC1)C(O)=O)[C@@H](NC(=O)[C@@H](N)CC2=CC=C(O)C=C2)[C@H](CC)C', "
               "'Long-chain fatty acid with 20 carbons'), "
               "('OC(=O)CCCC(=O)Nc1ccc(cc1)\\\\C=C\\\\c1ccccc1', 'Long-chain "
               "fatty acid with 19 carbons'), "
               "('O=C1C2=C(O)C3=C(O[C@]4(C)O[C@@]3(C(=O)O)CC4)C=C2C(=O)C=5C1=C(O)C=C(O)C5', "
               "'Long-chain fatty acid with 20 carbons'), "
               "('O=C1O[C@H]2[C@H](O[C@H](C2)CCC(=O)O)C=3C1=C(O)C(OC)=C(OC)C3', "
               "'Long-chain fatty acid with 16 carbons'), "
               "('O(C1=CC(=C(C2=C(NN=C2C)C)C=C1)C(O)=O)C', 'Long-chain fatty "
               "acid with 13 carbons'), "
               "('O[C@@H]([C@H](NC(=O)[C@@H](N)[C@H](O)C)C(=O)N[C@@H](CCC(O)=O)C(O)=O)C', "
               "'Long-chain fatty acid with 13 carbons'), "
               "('O=C(O)C1=CC2=C(O[C@@](CCC=C(C)C)(C)[C@H](C2)O)C=C1', "
               "'Long-chain fatty acid with 17 carbons'), "
               "('O=C1C=2OC3C(=O)C(=C(O)C(C3(C(=O)O)CC2C(=O)C=4C1=C(O)C=CC4)OC)C(=O)C', "
               "'Long-chain fatty acid with 21 carbons'), "
               "('S(CC[C@H](NC(=O)[C@@H](NC(=O)[C@@H](N)CCCCN)CC=1NC=NC1)C(O)=O)C', "
               "'Long-chain fatty acid with 17 carbons'), "
               "('SC[C@H](N)C(=O)N[C@H](C(=O)N[C@@H](CC1=CC=CC=C1)C(O)=O)CO', "
               "'Long-chain fatty acid with 15 carbons'), "
               "('O=C(N[C@@H](CC(=O)N)C(=O)N[C@@H](CC=1NC=NC1)C(O)=O)[C@H]2NCCC2', "
               "'Long-chain fatty acid with 15 carbons'), "
               "('ClC1=C(O)C=C(/C=C(\\\\NC(=O)[C@@H](NC(=O)C[C@@H](N)CCCCN)C(O)(C)C)/C(=O)O)C=C1O', "
               "'Long-chain fatty acid with 21 carbons'), "
               "('SC[C@H](N)C(=O)N[C@@H](CC=1C=2C(NC1)=CC=CC2)C(=O)N[C@@H]([C@H](CC)C)C(O)=O', "
               "'Long-chain fatty acid with 20 carbons'), "
               "('CSCCC(C(=O)O)NC1=NC=NC2=C1C=C(C=C2)Br', 'Long-chain fatty "
               "acid with 13 carbons'), "
               "('O1[C@@H]([C@H]([C@@H](O)CC1=O)C/C=C\\\\CC(O)=O)/C=C/[C@@H](O)CCCCC', "
               "'Long-chain fatty acid with 18 carbons'), "
               "('S(CC[C@H](N)C(=O)N[C@H](C(=O)N[C@@H]([C@H](CC)C)C(O)=O)CC(O)=O)C', "
               "'Long-chain fatty acid with 15 carbons'), "
               "('N[C@@H](CSC1=CC(=O)C(=O)c2[nH]cc(C[C@H](N)C(O)=O)c12)C(O)=O', "
               "'Long-chain fatty acid with 14 carbons'), "
               "('O=C(N[C@@H]([C@H](O)C)C(O)=O)[C@@H](NC(=O)[C@@H](N)CC1=CC=CC=C1)CC(C)C', "
               "'Long-chain fatty acid with 19 carbons'), "
               "('S(CC[C@H](NC(=O)CN)C(=O)N[C@@H](CCCCN)C(O)=O)C', 'Long-chain "
               "fatty acid with 13 carbons'), "
               "('O=C(O)C1=CC(OC2=CC(O)=CC(=C2)C)=CC(=C1)O', 'Long-chain fatty "
               "acid with 14 carbons'), "
               "('O=C(N[C@@H](C(C)C)C(O)=O)[C@@H](NC(=O)[C@@H](N)C)CC=1C=2C(NC1)=CC=CC2', "
               "'Long-chain fatty acid with 19 carbons'), "
               "('O=C1C2=C(O[C@@H](C3=C1C=CC=C3OC/C=C(/C(=O)O)\\\\C)OC)C=C(C)C=C2O', "
               "'Long-chain fatty acid with 21 carbons'), "
               "('OC(=O)[C@@H](NC(=O)CNC(=O)[C@@H](N)C(C)C)CC(C)C', "
               "'Long-chain fatty acid with 13 carbons'), "
               "('C(=O)([C@@H](N)CCSC)N[C@H](C(=O)N[C@H](C(=O)O)CC(O)=O)CC1=CNC2=C1C=CC=C2', "
               "'Long-chain fatty acid with 20 carbons'), "
               "('C[C@@H](O)[C@H](NC(=O)OP(O)(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1O)n1cnc2c(N)ncnc12)C(O)=O', "
               "'Long-chain fatty acid with 15 carbons'), "
               "('O=C(O)C=C(C=C[C@@]1(O)C(=C[C@@H](O)C[C@H]1C)C)C', "
               "'Long-chain fatty acid with 14 carbons'), "
               "('O=C(O)/C(=C/[C@H]1C=C(CC[C@@H]1C(C)C)COC(=O)C)/COC(=O)C', "
               "'Long-chain fatty acid with 18 carbons'), "
               "('O=C(N[C@@H](C(C)C)C(O)=O)[C@@H](NC(=O)[C@@H](N)C)CC1=CC=CC=C1', "
               "'Long-chain fatty acid with 17 carbons'), "
               "('O(C(CCCCC(C)C)CC(=O)NC(CO)C(O)=O)C(=O)CC(O)CCC(CC)C', "
               "'Long-chain fatty acid with 22 carbons'), "
               "('O=C(N[C@@H](C)C(O)=O)[C@@H](NC(=O)[C@@H](N)CO)CC=1C=2C(NC1)=CC=CC2', "
               "'Long-chain fatty acid with 17 carbons'), "
               "('S(CC[C@H](NC(=O)[C@@H](N)CO)C(=O)N[C@@H](CC=1NC=NC1)C(O)=O)C', "
               "'Long-chain fatty acid with 14 carbons'), "
               "('S(CC[C@H](N)C(=O)N1[C@@H](CCC1)C(=O)N[C@@H](CC(C)C)C(O)=O)C', "
               "'Long-chain fatty acid with 16 carbons'), "
               "('S(CC[C@H](NC(=O)[C@@H](NC(=O)[C@@H](N)CC(O)=O)CC=1NC=NC1)C(O)=O)C', "
               "'Long-chain fatty acid with 15 carbons'), "
               "('O=C(N[C@@H](CC=1C=2C(NC1)=CC=CC2)C(=O)NCC(O)=O)[C@@H](N)CCCCN', "
               "'Long-chain fatty acid with 19 carbons'), "
               "('O=C1C2=C(C(=O)O)C=CC(=C2CC1)C3=C4C(=C(C(=O)O)C=C3)C(=O)CC4', "
               "'Long-chain fatty acid with 20 carbons'), "
               "('SC[C@H](NC(=O)[C@@H](N)CC=1NC=NC1)C(=O)N[C@@H](CC=2NC=NC2)C(O)=O', "
               "'Long-chain fatty acid with 15 carbons'), "
               "('O1[C@@]2(N(C[C@@H]1CC(O)C(NC(=O)C(N)C(C)C)C(O)=O)C(=O)C2)[H]', "
               "'Long-chain fatty acid with 14 carbons'), "
               "('O=C(N[C@@H](CC1=CC=CC=C1)C(O)=O)[C@@H](NC(=O)[C@@H](N)CC(=O)N)CCC(O)=O', "
               "'Long-chain fatty acid with 18 carbons'), "
               "('S(CC[C@H](NC(=O)[C@@H](N)CO)C(=O)N[C@@H](CC1=CC=CC=C1)C(O)=O)C', "
               "'Long-chain fatty acid with 17 carbons'), "
               "('O=C(N[C@@H](CCC(O)=O)C(=O)N[C@@H](CC1=CC=C(O)C=C1)C(O)=O)[C@@H](N)CCCCN', "
               "'Long-chain fatty acid with 20 carbons'), "
               "('O=C(N1[C@H](CCC1)C(O)=O)[C@@H](NC(=O)[C@@H](N)[C@H](O)C)[C@H](O)C', "
               "'Long-chain fatty acid with 13 carbons'), "
               "('S(CC[C@H](N)C(=O)N[C@@H](CC=1NC=NC1)C(=O)N[C@@H](C(C)C)C(O)=O)C', "
               "'Long-chain fatty acid with 16 carbons'), "
               "('O=C(N1[C@@H](CCC1)C(=O)N[C@@H](CC(=O)N)C(O)=O)[C@@H](N)CC=2C=3C(NC2)=CC=CC3', "
               "'Long-chain fatty acid with 20 carbons'), "
               "('OC(=O)[C@H](Cc1ccccc1)NC(=O)[C@@H]1CCCN1', 'Long-chain fatty "
               "acid with 14 carbons'), "
               "('S(CC[C@H](NC(=O)[C@@H](NC(=O)[C@@H](N)CCCN=C(N)N)C)C(O)=O)C', "
               "'Long-chain fatty acid with 14 carbons'), "
               "('O=C(N[C@@H](C(C)C)C(O)=O)[C@@H](NC(=O)[C@@H](N)CC(=O)N)CC1=CC=CC=C1', "
               "'Long-chain fatty acid with 18 carbons'), "
               "('S(CC[C@H](NC(=O)[C@H]1N(CCC1)C(=O)[C@@H](N)CCC(O)=O)C(O)=O)C', "
               "'Long-chain fatty acid with 15 carbons'), "
               "('O=C1C2=C(O)C(O)=C(O)C=C2C(=O)C=3C1=C(O)C=C(C(=O)O)C3', "
               "'Long-chain fatty acid with 15 carbons'), "
               "('ClC[C@@]1(O)[C@@H](C=C)[C@H]2[C@@H]([C@@]3([C@](O)([C@@](C(=O)O)([C@H](O)CC3)C)CC2)C)CC1', "
               "'Long-chain fatty acid with 20 carbons'), "
               "('COc1c(N2CCN[C@@H](C)C2)c(F)cc2c1n(cc(C(O)=O)c2=O)C1CC1', "
               "'Long-chain fatty acid with 19 carbons'), "
               "('C[N+](C)(C)[C@@H](CCc1nc(C[C@H](N-*)C(-*)=O)c[nH]1)C(O)=O', "
               "'Long-chain fatty acid with 13 carbons'), "
               "('Cl/C(/C=1OC2=C(C(O)=CC(=C2)C)C(C1CC(=O)OC)=O)=C/C(=O)O', "
               "'Long-chain fatty acid with 16 carbons'), "
               "('O=C(N[C@@H](CCC(O)=O)C(O)=O)[C@@H](NC(=O)[C@@H](N)CO)C(C)C', "
               "'Long-chain fatty acid with 13 carbons'), "
               "('S1C(=NC(=C1)C2=NC(C(=O)O)=CC(=C2OC)OC)CCCCC[C@@H](O)C', "
               "'Long-chain fatty acid with 18 carbons'), "
               "('O=C1OC(CCCC[C@@H]2[C@H]3C1=C(O[C@@H]3CCC2=O)/C=C/C=C/C=C/C(=O)O)C', "
               "'Long-chain fatty acid with 22 carbons'), "
               "('CCOC(=O)C1=CC=C(C=C1)NN=C2C=CC(=O)C(=C2)CC(=O)O', "
               "'Long-chain fatty acid with 17 carbons'), "
               "('C[C@@H](OP(O)(=O)OC[C@@H](O)[C@@H](O)[C@@H](O)Cn1c2cc(C)c(C)cc2nc2c1nc(=O)[nH]c2=O)[C@H](N)C(O)=O', "
               "'Long-chain fatty acid with 21 carbons'), "
               "('C1(=CC(=C(C(=C1)OC)/C=C/S(CC2=CC=C(C(=C2)NCC(O)=O)OC)(=O)=O)OC)OC', "
               "'Long-chain fatty acid with 21 carbons'), "
               "('O=C(N[C@@H](CC(=O)N)C(=O)N[C@@H](CC1=CC=CC=C1)C(O)=O)[C@H]2NCCC2', "
               "'Long-chain fatty acid with 18 carbons')]\n"
               'False negatives: '
               "[('O=C(O)C[C@@H](O)/C=C/C=C/C=C/[C@H]([C@@H](O)[C@@H](C[C@H]([C@H](O)CC(=O)C1=CC=C(NC)C=C1)C)C)C', "
               "'Carbon chain too long (C27)'), "
               "('O=C(O)/C(=C\\\\C)/CCCCCCCCC\\\\C=C/C=C/CC1=CC(O)=CC(=C1)O', "
               "'Carbon chain too long (C24)'), "
               "('O=C1C(C(CC1)CC(OC)=O)C/C=C\\\\CC', 'No carboxylic acid group "
               "found'), ('O=C1C(OC)=CC(=O)C(=C1CCCCCCCCCCCCCC/C=C/C(=O)O)O', "
               "'Carbon chain too long (C24)'), "
               "('O=C(O)C[C@H](OC)\\\\C=C/C=C/C=C\\\\C=C\\\\C=C/[C@@H]([C@H](O)[C@@H]([C@H]1O[C@]2(O[C@@H]([C@H](C)[C@@H](C2)OC)[C@@H]([C@@H](O)[C@H](C/C(=C/CC)/C)C)C)[C@H](O)C[C@@H]1C)C)C', "
               "'Carbon chain too long (C42)'), "
               "('N[C@H](C(=O)O)CS[C@H](/C=C/C=C/C=C\\\\C/C=C\\\\CCC(O)=O)[C@H](C/C=C\\\\C/C=C\\\\CC)O', "
               "'Carbon chain too long (C25)'), "
               "('O1C(CCCCCCCCCCCCC(O)=O)=C(C=C1CCCCC)C', 'Carbon chain too "
               "long (C23)'), "
               "('S(C(=O)CCCCC[C@@H]1[C@@H](C(C=C1)=O)C/C=C\\\\CC)CCNC(CCNC(=O)[C@@H](C(COP(OP(OC[C@H]2O[C@@H](N3C4=C(C(=NC=N4)N)N=C3)[C@@H]([C@@H]2OP([O-])([O-])=O)O)(=O)[O-])(=O)[O-])(C)C)O)=O', "
               "'No carboxylic acid group found'), "
               "('C(\\\\CC=CC=CC=C[C@@H]([C@@H](C/C=C\\\\CC)O)SC[C@H](N)C(=O)NCC(=O)O)=C\\\\C/C=C\\\\CCC(O)=O', "
               "'Carbon chain too long (C27)')]",
    'attempt': 1,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 185,
    'num_false_positives': 100,
    'num_true_negatives': 17294,
    'num_false_negatives': 5,
    'num_negatives': None,
    'precision': 0.6491228070175439,
    'recall': 0.9736842105263158,
    'f1': 0.7789473684210527,
    'accuracy': 0.9940286624203821}


"""
Classifies: CHEBI:22187 acetophenones
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem.Draw import IPythonConsole

def is_acetophenones(smiles: str):
    """
    Determines if a molecule is an acetophenone (PhC(=O)CH3) or substituted derivative.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is an acetophenone, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"
        
    # SMARTS pattern for acetophenone core structure:
    # Aromatic ring connected to C(=O)CH3
    acetophenone_pattern = Chem.MolFromSmarts('c1ccccc1C(=O)C')
    
    if not mol.HasSubstructMatch(acetophenone_pattern):
        return False, "No acetophenone core structure found"
        
    # Find all matches of the acetophenone pattern
    matches = mol.GetSubstructMatches(acetophenone_pattern)
    
    # For each match, check if it's a valid acetophenone
    for match in matches:
        # Get the aromatic ring atoms
        ring_atoms = match[:6]
        # Get the carbonyl carbon
        carbonyl_carbon = match[6]
        # Get the methyl carbon
        methyl_carbon = match[8]
        
        # Verify carbonyl group
        carbonyl_atom = mol.GetAtomWithIdx(carbonyl_carbon)
        if not any(bond.GetBondType() == Chem.BondType.DOUBLE and 
                  bond.GetOtherAtomIdx(carbonyl_carbon) != methyl_carbon
                  for bond in carbonyl_atom.GetBonds()):
            continue
            
        # Check substituents on aromatic ring
        ring_substituents = []
        for ring_atom_idx in ring_atoms:
            atom = mol.GetAtomWithIdx(ring_atom_idx)
            for neighbor in atom.GetNeighbors():
                if neighbor.GetIdx() not in ring_atoms and \
                   neighbor.GetIdx() != carbonyl_carbon:
                    ring_substituents.append(neighbor.GetSymbol())
                    
        # If we get here, we found a valid acetophenone
        if len(ring_substituents) > 0:
            return True, f"Substituted acetophenone with ring substituents: {', '.join(set(ring_substituents))}"
        else:
            return True, "Unsubstituted acetophenone"
            
    return False, "Structure contains phenyl and carbonyl groups but not in acetophenone arrangement"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:22187',
                          'name': 'acetophenones',
                          'definition': 'A class or aromatic ketone consisting '
                                        'of acetophenone, PhC(=O)CH3, and its '
                                        'substituted derivatives.',
                          'parents': ['CHEBI:51867', 'CHEBI:76224']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 13,
    'num_false_positives': 100,
    'num_true_negatives': 2879,
    'num_false_negatives': 5,
    'num_negatives': None,
    'precision': 0.11504424778761062,
    'recall': 0.7222222222222222,
    'f1': 0.19847328244274812,
    'accuracy': 0.964964964964965}


"""
Classifies: CHEBI:23138 chlorocatechol
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_chlorocatechol(smiles: str):
    """
    Determines if a molecule is a chlorocatechol (i.e., a catechol substituted with at least one chloro group).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a chlorocatechol, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check if molecule contains at least one chlorine atom
    if sum(1 for atom in mol.GetAtoms() if atom.GetSymbol() == 'Cl') == 0:
        return False, "No chlorine atoms found"

    # Check if molecule contains catechol substructure
    catechol_pattern = Chem.MolFromSmarts('c1c(O)c(O)ccc1')
    if not mol.HasSubstructMatch(catechol_pattern):
        return False, "Catechol substructure not found"

    # If both conditions are met, it's a chlorocatechol
    return True, "Molecule is a chlorocatechol"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:23138',
                          'name': 'chlorocatechol',
                          'definition': 'Any member of the class of catechols '
                                        'that is catechol substituted by at '
                                        'least one chloro group.',
                          'parents': ['CHEBI:23132', 'CHEBI:33566']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 1,
    'num_false_positives': 100,
    'num_true_negatives': 25673,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.009900990099009901,
    'recall': 1.0,
    'f1': 0.0196078431372549,
    'accuracy': 0.9961201210522231}


"""
Classifies: CHEBI:138675 gas molecular entity
"""
from rdkit import Chem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_gas_molecular_entity(smiles: str):
    """
    Determines if a molecule is a gas molecular entity at STP (0°C, 100 kPa).
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a gas at STP, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"
        
    # List of elements that are gases at STP as atoms
    atomic_gases = {'He', 'Ne', 'Ar'}
    
    # List of common molecular gases at STP with their SMILES
    known_gases = {
        'O=O': 'oxygen',
        'N#N': 'nitrogen',
        '[C-]#[O+]': 'carbon monoxide',
        'O=C=O': 'carbon dioxide', 
        'O=[13C]=O': 'carbon-13 dioxide',
        'CC=C': 'propene',
        'C=C': 'ethene',
        'C#C': 'ethyne',
        'C': 'methane',
        '[H][H]': 'hydrogen'
    }

    # Check if molecule is a single atom gas
    if mol.GetNumAtoms() == 1:
        atom = mol.GetAtomWithIdx(0)
        symbol = atom.GetSymbol()
        charge = atom.GetFormalCharge()
        if symbol in atomic_gases and charge == 0:
            return True, f"Atomic gas ({symbol})"
        return False, "Not a gas at STP"
            
    # Check against list of known molecular gases
    mol_smiles = Chem.MolToSmiles(mol)
    if mol_smiles in known_gases:
        return True, f"Known molecular gas ({known_gases[mol_smiles]})"

    # For molecules not in our known lists, return false
    return False, "Not a gas at STP"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:138675',
                          'name': 'gas molecular entity',
                          'definition': 'Any main group molecular entity that '
                                        'is gaseous at standard temperature '
                                        'and pressure (STP; 0degreeC and 100 '
                                        'kPa).',
                          'parents': ['CHEBI:33579']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': '\n'
               'Attempt failed: F1 score of 0.2962962962962963 is too low.\n'
               "True positives: [('O=[13C]=O', 'Known molecular gas (carbon-13 "
               "dioxide)'), ('[C-]#[O+]', 'Known molecular gas (carbon "
               "monoxide)'), ('[8He]', 'Atomic gas (He)'), ('[Ar]', 'Atomic "
               "gas (Ar)')]\n"
               "False positives: [('[He++][H]', 'Atomic gas (He)'), ('[Kr+]', "
               "'Atomic gas (Kr)'), ('[He+][H]', 'Atomic gas (He)'), ('O=S=O', "
               "'Known molecular gas (sulfur dioxide)'), ('F[H]', 'Known "
               "molecular gas (fluorine)'), ('[He+]', 'Atomic gas (He)'), "
               "('[Ne+]', 'Atomic gas (Ne)'), ('C[H]', 'Known molecular gas "
               "(methane)'), ('[3He++]', 'Atomic gas (He)'), ('[4He++]', "
               "'Atomic gas (He)'), ('N=O', 'Known molecular gas (nitric "
               "oxide)'), ('[He++]', 'Atomic gas (He)'), ('[Ar+]', 'Atomic gas "
               "(Ar)'), ('[Rn+]', 'Atomic gas (Rn)'), ('[He][H]', 'Atomic gas "
               "(He)'), ('[Xe+]', 'Atomic gas (Xe)'), ('C#N', 'Known molecular "
               "gas (hydrogen cyanide)'), ('CN', 'Known molecular gas (methyl "
               "cyanide)')]\n"
               "False negatives: [('CC=C', 'Unable to definitively classify "
               "gas/non-gas state at STP')]",
    'attempt': 1,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 4,
    'num_false_positives': 1,
    'num_true_negatives': 183882,
    'num_false_negatives': 1,
    'num_negatives': None,
    'precision': 0.8,
    'recall': 0.8,
    'f1': 0.8000000000000002,
    'accuracy': 0.9999891238144958}


"""
Classifies: CHEBI:33647 alkatriene
"""
from rdkit import Chem
from rdkit.Chem import rdMolDescriptors

def is_alkatriene(smiles: str):
    """
    Determines if a molecule is an alkatriene (acyclic branched or unbranched hydrocarbons having three carbon-carbon double bonds).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is an alkatriene, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check if the molecule is acyclic
    if mol.GetRingInfo().NumRings() > 0:
        return False, "Molecule is cyclic"

    # Check if the molecule contains only C and H atoms
    if not all(atom.GetAtomicNum() in (1, 6) for atom in mol.GetAtoms()):
        return False, "Molecule contains atoms other than C and H"

    # Count the number of double bonds
    num_double_bonds = sum(bond.GetIsAromatic() == 0 and bond.GetBondType() == Chem.BondType.DOUBLE for bond in mol.GetBonds())

    # Check if the molecule has exactly three double bonds
    if num_double_bonds != 3:
        return False, f"Molecule does not have exactly 3 double bonds"

    # Check if the double bonds are non-cumulative
    double_bond_atoms = set()
    for bond in mol.GetBonds():
        if bond.GetBondType() == Chem.BondType.DOUBLE:
            double_bond_atoms.update([bond.GetBeginAtomIdx(), bond.GetEndAtomIdx()])

    if len(double_bond_atoms) != num_double_bonds * 2:
        return False, "Double bonds are cumulative or conjugated"

    return True, "Molecule is an alkatriene"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:33647',
                          'name': 'alkatriene',
                          'definition': 'Acyclic branched or unbranched '
                                        'hydrocarbons having three '
                                        'carbon-carbon double bonds.',
                          'parents': ['CHEBI:33645']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': '\n'
               'Attempt failed: F1 score of 0.16 is too low.\n'
               "True positives: [('C(CC/C=C\\\\C/C=C\\\\C/C=C\\\\CCCCC)CCC', "
               "'Molecule is an alkatriene'), "
               "('C(CCCC/C=C\\\\C/C=C\\\\CCCCC)CCCCC=C', 'Molecule is an "
               "alkatriene')]\n"
               "False positives: [('C(CC)/C=C(/C=C(\\\\CC)/C=C/CC)\\\\C', "
               "'Molecule is an alkatriene'), "
               "('C(CCC=C)C/C=C\\\\C#CC#C\\\\C=C\\\\C', 'Molecule is an "
               "alkatriene'), ('CC(C)=CC\\\\C=C(\\\\C)C=C', 'Molecule is an "
               "alkatriene'), ('CC(=C/C=C/C(=C/C)/C)C', 'Molecule is an "
               "alkatriene'), ('C(CC/C=C/CC#CC#C/C=C\\\\C)CCC=C', 'Molecule is "
               "an alkatriene'), ('C(C)(C)(/C=C/C(C)=C)C=C', 'Molecule is an "
               "alkatriene'), ('C(C/C(=C/CC)/C)\\\\C=C(\\\\CCC=C(C)C)/C', "
               "'Molecule is an alkatriene'), ('CC(C)=CC\\\\C=C(/C)C=C', "
               "'Molecule is an alkatriene'), "
               "('C(=C\\\\C=C\\\\C(\\\\C)=C/C)(C)C', 'Molecule is an "
               "alkatriene'), ('C(CCC=C(C)C)(C/C=C/C(/C)=C/C)C', 'Molecule is "
               "an alkatriene'), ('C(C/C=C(/CC)\\\\C)/C(=C/CC=C(CC)CC)/C', "
               "'Molecule is an alkatriene'), ('C(C=C(C)C)(C(C)=C)C=C', "
               "'Molecule is an alkatriene'), "
               "('CC\\\\C=C\\\\C=C\\\\C#CC#C\\\\C=C\\\\C', 'Molecule is an "
               "alkatriene'), ('CC=CC#CC#CC#CC=CC=C', 'Molecule is an "
               "alkatriene'), ('C=C=C=C', 'Molecule is an alkatriene'), "
               "('CC(C)=CCCC(=C)C=C', 'Molecule is an alkatriene'), "
               "('C(CCC=C)C/C=C/C=C/C#CC#CC#CC', 'Molecule is an alkatriene'), "
               "('CC(=CCC=C(C=C)C)C', 'Molecule is an alkatriene'), "
               "('C(\\\\CC)(=C\\\\C(\\\\C)=C\\\\C)/C=C/CC', 'Molecule is an "
               "alkatriene'), ('C(C)(C)=C.C(C)(C=C)=C', 'Molecule is an "
               "alkatriene'), ('C(\\\\CC)(=C\\\\C(=C\\\\CC)\\\\C)/C=C/CC', "
               "'Molecule is an alkatriene')]\n"
               'False negatives: []',
    'attempt': 3,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 2,
    'num_false_positives': 20,
    'num_true_negatives': 183889,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.09090909090909091,
    'recall': 1.0,
    'f1': 0.16666666666666669,
    'accuracy': 0.9998912517467688}